/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tcl_scripting.h"
#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#include <qmenubar.h>
#include <qinputdialog.h>

#include "ignore.h"
#include "events.h"
#include "config_file.h"
#include "config_dialog.h"
#include "hints.h"
#include "status.h"
#include "misc.h"
#include "message_box.h"
#include "emoticons.h"
#include "search.h"
#include "modules.h"
#include "history.h"

// External modules
#include "../sound/sound.h"
#include "../docking/docking.h"

#ifdef USE_TK
#include <tk.h>
#endif

/* Tcl Command quit
SYNTAX: quit ?opis?
DESCRIPTION:
Zapisuje ustawienia i list kontaktw, nastepnie zamyka program. Opcjonalnie mona
poda opis. W takim wypadku przed zamkniciem programu ustawiany jest status
'Niedostpny z opisem' wraz z podanym opisem.
*/
int Tcl_CmdQuit(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "quit ?description?");
        return TCL_ERROR;
    }
    if (argc > 1)
    {
        own_description = argv[1];
        kadu->setStatus(GG_STATUS_NOT_AVAIL_DESCR);
    }
    else
    {
        kadu->setStatus(GG_STATUS_NOT_AVAIL);
    }
    gadu->logout();
    qApp->exit(0);
    return TCL_OK;
}

/* Tcl Command connect
SYNTAX: connect
DESCRIPTION:
czy Kadu z serwerem GG i ustawia status 'Dostpny'.
*/
int Tcl_CmdConnect(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "connect");
        return TCL_ERROR;
    }
    kadu->setStatus(GG_STATUS_AVAIL);
    return TCL_OK;
}

/* Tcl Command disconnect
SYNTAX: disconnect
DESCRIPTION:
Odcza Kadu od serwera GG.
*/
int Tcl_CmdDisconnect(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "disconnect");
        return TCL_ERROR;
    }
    gadu->logout();
    AutoConnectionTimer::off();
    kadu->setCurrentStatus(GG_STATUS_NOT_AVAIL);
    return TCL_OK;
}

/* Tcl Command msgbox
SYNTAX: msgbox typ ?tytu? wiadomo ?opcje?

DESCRIPTION:
Wywietla okno z wiadomoci (w ktrej mona uywa znacznikw HTML), o podanym tytule.
Typ moe przyj jedn z siedmiu wartoci: info, warning, critical, msg, ask, status, oraz progress.

SUBSYNTAX: msgbox info/warning/critical tytu wiadomo
DESCRIPTION:
Standardowe okna dialogowe biblioteki QT. Zawieraj odpowiedni ikonk, oraz mona dla nich
zdefiniowa tytu okna.

SUBSYNTAX: msgbox msg wiadomo
DESCRIPTION:
Najprostsze okno informacyjne z podan <i>wiadomoci</i>, oraz przyciskiem "OK".

SUBSYNTAX: msgbox ask wiadomo
DESCRIPTION:
Okno z <i>wiadomoci</i> jako pytaniem, na ktre uytkownik moe odpowiedzie
poprzez kliknicie na jednym z 2 przyciskw - "tak" lub "nie". Dokonany wybr decyduje
o zwrconej przez komend wartoci - 1 (tak) lub 0 (nie).

SUBSYNTAX: msgbox status wiadomo ?-close?
DESCRIPTION:
Okno informujce uytkownika o wykonywanej aktualnie jakiej czynnoci. Nie ma w nim
adnych przyciskw, a zamykane jest przez t sam komend (z t sam <i>wiadomoci</i>)
z opcj -close, lub odpowiedni funkcj menadera okien.

SUBSYNTAX: msgbox progress wiadomo ?kroki? ?-progress warto? ?-close?
DESCRIPTION:
Informuje uytkownika o wykonywanej czynnoci, jednoczenie pokazujc stopie
zaawansowania w przebiegu tej czynnoci. Przy tworzeniu okna wymagany jest argument <i>krokw</i>,
ktry decyduje, ile krokw jest potrzebnych do wykonania czynnoci. Kadorazowo, gdy owa czynno
przejdzie do nastpnego kroku, naley uruchomi t komend z argumentem tej samej <i>wiadomoci</i>,
pomijajc argument <i>krokw</i>, za to podajc opcj -progress i odpowiedni dla niej warto,
ktra okrela z kolei, w ktrym <i>kroku</i> aktualnie jest wykonywana czynno. Po zakoczeniu
pracy okno naley zamkn t sam komend z opcj -close. Dodatkowo w oknie tym jest przycisk
"anuluj", ktry umoliwia przerwanie wykonywania czynnoci przez uytkownika. Wywoywane jest wtedy
zdarzenie <b>progresscancel</b> (patrz: komenda <a href=on.html></b>on<b></a>), w implementacji
ktrego powinna zosta przerwana wykonywana czynno.
*/
int Tcl_CmdMessageBox(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 3)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "msgbox type ?caption? message ?options?");
        return TCL_ERROR;
    }
    QStringList types = QStringList::split(" ", "critical information warning msg ask status progress");
    switch (types.findIndex(QString(argv[1])))
    {
        case 0:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "msgbox critical caption message");
                return TCL_ERROR;
            }
            QMessageBox::critical( 0, QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]) );
            break;
        }
        case 1:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "msgbox information caption message");
                return TCL_ERROR;
            }
            QMessageBox::information( 0, QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]) );
            break;
        }
        case 2:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "msgbox warning caption message");
                return TCL_ERROR;
            }
            QMessageBox::warning( 0, QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]) );
            break;
        }
        case 3:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "msgbox msg message");
                return TCL_ERROR;
            }
            MessageBox::msg(QString::fromUtf8(argv[2]));
            break;
        }
        case 4:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "msgbox ask message");
                return TCL_ERROR;
            }
            Tcl_SetResult(interp, (char*)QString::number(
                MessageBox::ask(QString::fromUtf8(argv[2]))
                ).latin1(), TCL_VOLATILE);
            break;
        }
        case 5:
        {
            if (argc < 3 || argc > 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "msgbox status message ?-close?");
                return TCL_ERROR;
            }
            if (argc == 3)
            {
                MessageBox::status(QString::fromUtf8(argv[2]));
            }
            else
            {
                if (QString::compare(argv[3], "-close"))
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, "msgbox status message ?-close?");
                    return TCL_ERROR;
                }
                MessageBox::close(QString::fromUtf8(argv[2]));
            }
            break;
        }
        case 6:
        {
            if (argc < 3 || argc > 5)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "msgbox progress message ?steps? ?-progress value? ?-close?");
                return TCL_ERROR;
            }
            if (argc == 5)
            {
                bool ok;
                int val = QString(argv[4]).toInt(&ok);
                if (QString::compare(argv[3], "-progress") || !ok)
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, "msgbox progress message -progress value");
                    return TCL_ERROR;
                }
                MessageBox::progress(QString::fromUtf8(argv[2]), val);
            }
            else if (argc == 4)
            {
                if (!QString::compare(argv[3], "-close"))
                {
                    MessageBox::close(QString::fromUtf8(argv[2]));
                }
                else
                {
                    bool ok;
                    int val = QString(argv[3]).toInt(&ok);
                    if (!ok)
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, "msgbox progress message ?steps? ?-progress value? ?-close?");
                        return TCL_ERROR;
                    }
                    int sessID = *((int*) clientData);
                    TclSession* tsess = tclSess[sessID];
                    MessageBox::progress(QString::fromUtf8(argv[2]), tsess->tclEvtHnd(), SLOT(msgBoxCancel()), val);
                }
            }
            else
            {
                MessageBox::progress(QString::fromUtf8(argv[2]));
            }
            break;
        }
        default:
        {
            QString err = QString("type should be one of: %1").arg(types.join(", "));
            Tcl_WrongNumArgs(interp, 0, NULL, err.local8Bit());
            return TCL_ERROR;
        }
    }
    return TCL_OK;
}

/* Tcl Command inputdialog
GLOBALSYNTAX: inputdialog typ tytu wiadomo ?argumenty?

DESCRIPTION:
Wywietla okno z polem do wpisania danych przez uytkownika. Tytu okna bdzie taki,
jak podany przez uytkownika. Nad polem pojawi si podana wiadomo (mona w niej uywa znacznikw HTML).
Okno ma 2 przyciski - OK, oraz Anuluj. Jeli naciniety zostanie przycisk Anuluj, to komenda ta zwrci pusty acuch,
niezalenie od wpisanej danej w polu, natomiast jeli przycisk OK zostanie nacinity,
wtedy zwrcona zostanie zawarto pola.

SUBSYNTAX: inputdialog text tytu wiadomo ?domylnyTekst ?typWywietlania??

DESCRIPTION:
Opcjonalnie mona poda <i>domylny tekst</i>, oraz <i>rodzaj wywietlania</i>, ktry moe przyj
jedn z trzech wartoci: <b>normal</b>, <b>noecho</b>, <b>password</b>, gdzie pierwszy jest uywany
domylnie, drugi powoduje, e pomimo i uytkownik wpisuje tekst, w polu nie pokazuj si adne znaki.
Wreszcie trzecia z moliwoci wywietla kady wpisany znak jako gwiazdk.

SUBSYNTAX: inputdialog item tytu wiadomo lista ?pocztkowaPozycja ?edytowalny??

DESCRIPTION:
Dodatkowym wymaganym argumentem jest lista sw, ktre zo si na liste pozycji do wyboru.
Opcjonalny argument to <i>pocztkowa pozycja</i>, ktr stanowi liczba wiksza, lub rwna 0.
Warto rwna 0 oznacza, e pierwsze sowo z listy bdzie domylnie wybrane, kade nastpne sowo
oznaczone jest kolejn liczb. Kolejnym opcjonalnym argumentem jest ?edytowalno? pola. Domylnie
jego warto jest rwna TRUE, co oznacza, e oprcz sw z podanej listy, uytkownik moe poda
swoje wasne. Jeli argument ten zostanie ustawiony na warto negatywn (tj. false, no, off, 0),
to takiej moliwoci nie bdzie.

SUBSYNTAX: inputdialog iteger tytu wiadomo ?wartoDomylna ?minimum ?maksimum ?krok????

DESCRIPTION:
Jako pierwszy argument opcjonalny mona poda <i>domyln warto</i> pola. Komenda dotyczy typu danych integer,
w zwizku z tym musi to by warto liczbowa, cakowita. Kolejne argumenty opcjonalne to <i>minimalna</i> i <i>maksymalna</i>
warto, ktr bdzie mg poda uytkownik. Ostatni z argumentw okrela, jak duy ma by <i>krok</i>, czyli o ile
ma si zwikszy warto liczby w polu, po klikniciu na strzak wzrostu, a ile zmale, gdy na strzak zmniejszania.
Warto uywa tego dialogu, kiedy zaley nam, aby uytkownik mg wpisywa tylko liczby cakowite (ew. z rzdanego zakresu).

SUBSYNTAX: inputdialog double tytu wiadomo ?wartoDomylna ?minimum ?maksimum ?precyzja????

DESCRIPTION:
Okno dialogowe podobne do inputDialog integer, tyle e tutaj mona (a nawet trzeba) poda liczbe rzeczywista,
w postaci 'xxx.xxxxxx', gdzie liczba cyfr przed przecinkiem jest nieograniczona*, a po przecinku jest okrelona przez ostatni
z argumentw opcjonalnych. Podana precyzja dotyczy tylko wartoci, ktr moe wpisa uytkownik. Bez wzgldu na warto
<i>precyzji</i> zwrcona liczba bdzie miaa dokadno 6 miejsc po przecinku.
*/
int Tcl_CmdInputDialog(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 4)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "inputdialog type caption message ?arguments?");
        return TCL_ERROR;
    }
    QStringList types = QStringList::split(" ", "text item integer double");
    QString str = argv[1];
    switch (types.findIndex(str))
    {
        case -1:
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "valid types: text, item, integer, double");
            return TCL_ERROR;
            break;
        }
        case 0:
        {
            if (argc > 6)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "inputdialog text caption message ?defaultText ?echoType??");
                return TCL_ERROR;
            }
            QString def = "";
            QLineEdit::EchoMode type = QLineEdit::Normal;
            bool ok;
            if (argc > 4)
            {
                def = QString::fromUtf8(argv[4]);
                if (argc > 5)
                {
                    QString tmp = argv[5];
                    if (!tmp.compare("normal"))
                    {
                        type = QLineEdit::Normal;
                    }
                    else if (!tmp.compare("noecho"))
                    {
                        type = QLineEdit::NoEcho;
                    }
                    else if (!tmp.compare("password"))
                    {
                        type = QLineEdit::Password;
                    }
                    else
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, "inputdialog text caption message ?defaultText ?echoType??");
                        return TCL_ERROR;
                    }
                }
            }
            QString text = QInputDialog::getText(QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), type, def, &ok );
            if ( ok && !text.isEmpty() ) {
                Tcl_Obj* result;
                QCString utf = text.utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            }
            break;
        }
        case 1:
        {
            QString syntax = "inputdialog item caption message list ?current ?editable??";
            if (argc < 5)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                return TCL_ERROR;
            }
            int cnt = 1;
            bool edit = 1, conv, ok;
            if (argc > 5)
            {
                cnt = QString(argv[5]).toInt(&conv);
                if (!conv)
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                    return TCL_ERROR;
                }
                if (argc > 6)
                {
                    if (argc > 7)
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                        return TCL_ERROR;
                    }
                    int tmp;
                    Tcl_GetBoolean(interp, argv[6], &tmp);
                    edit = tmp;
                }
                else
                {
                    edit = 1;
                }
            }
            QStringList lst = QStringList::split(" ", QString::fromUtf8(argv[4]));

            QString text = QInputDialog::getItem(
                QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), lst, cnt, edit, &ok);

            if ( ok && !text.isEmpty() ) {
                Tcl_Obj* result;
                QCString utf = text.utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            }
            break;
        }
        case 2:
        {
            QString syntax = "inputDialog iteger caption message ?default ?min ?max ?step????";
            int def = 0;
            int min = -2147483647;
            int max = 2147483647;
            int step = 1;
            bool ok;
            if (argc > 4)
            {
                def = QString(argv[4]).toInt();
                if (argc > 5)
                {
                    min = QString(argv[5]).toInt();
                    if (argc > 6)
                    {
                        max = QString(argv[6]).toInt();
                        if (argc > 7)
                        {
                            step = QString(argv[7]).toInt();
                            if (argc > 8)
                            {
                                Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                                return TCL_ERROR;
                            }
                        }
                    }
                }
            }
            int res = QInputDialog::getInteger(QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), def, min, max, step, &ok );
            if ( ok )
                Tcl_SetResult(interp, (char*)QString::number(res).latin1(), TCL_VOLATILE);

            break;
        }
        case 3:
        {
            QString syntax = "inputDialog double caption message ?default ?min ?max ?precision????";
            int def = 0;
            int min = -2147483647;
            int max = 2147483647;
            int precision = 1;
            bool ok;
            if (argc > 4)
            {
                def = QString(argv[4]).toInt();
                if (argc > 5)
                {
                    min = QString(argv[5]).toInt();
                    if (argc > 6)
                    {
                        max = QString(argv[6]).toInt();
                        if (argc > 7)
                        {
                            precision = QString(argv[7]).toInt();
                            if (argc > 8)
                            {
                                Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                                return TCL_ERROR;
                            }
                        }
                    }
                }
            }
            double res = QInputDialog::getDouble(QString::fromUtf8(argv[2]), QString::fromUtf8(argv[3]), def, min, max, precision, &ok );
            if ( ok )
                Tcl_SetResult(interp, (char*)QString::number(res, 'f').latin1(), TCL_VOLATILE);

            break;
        }
    }
    return TCL_OK;
}

/* Tcl Command filedialog
SYNTAX: filedialog typ tytu ?nazwaFiltra maskaFiltra ?nazwaFiltra maskaFiltra ?...???
DESCRIPTION:
Wywietla okno przegldania plikw. Typ moe przyj jedn z trzech wartoci: 1) <b>open</b>, 2) <b>save</b>, lub 3) <b>getdir</b>,
ktre oznaczaj odopwiednio, e wybrany plik: 1) musi istnie, 2) moe, ale nie musi istnie, 3) Musi istnie i musi to by katalog.
Dodatkowo mona poda filtry, ktre pomog w wyszukiwaniu plikw. <i>Nazwa filtru</i> moe przyj warto np: "Obrazki", lub
"Pliki konfiguracyjne". Nie ma to wikszego znaczenia, bdzie to wycznie dla informacji uytkownika. <i>Maska filtra</i> natomiast
okrela jakie typy plikw powinny by wywietlane. Jest to lista sw oddzielonych spacjami, np: "*.jpg *.png", lub "*.cfg".
Dla kadej <i>nazwy filtra</i> musi by podana jako nastpna <i>maska</i> dla tego filtra. Jeli aden filtr nie zostanie
podany, domylnie zostanie uyty "Wszystkie pliki" "*".

EXAMPLE:
set file [filedialog open "Wybierz plik" "Pliki rdowe C/C++" "*.c *.cpp" "Skrypty TCL" "*.tcl"]
ENDOFEXAMPLE
*/
int Tcl_CmdFileDialog(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 3 || !(argc % 2))
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "filedialog type caption ?filtername filtermasks ?filtername filtermasks ?...???");
        return TCL_ERROR;
    }
    QString cdir = QDir::currentDirPath(), type = QString::fromUtf8(argv[1]),
            caption = QString::fromUtf8(argv[2]), tmp, res;
    QStringList types = QStringList::split(" ", "open save getdir"), filters;
    for ( int i = 3; i < argc; i++ )
    {
        tmp = argv[i];
        tmp.append(" (");
        i++;
        tmp.append(argv[i]);
        tmp.append(")");
        filters.append(tmp);
    }
    switch (types.findIndex(type))
    {
        case 0:
        {
            res = QFileDialog::getOpenFileName(QString::null, filters.join(";;"), 0, 0, caption);
            break;
        }
        case 1:
        {
            res = QFileDialog::getSaveFileName(QString::null, filters.join(";;"), 0, 0, caption);
            break;
        }
        case 2:
        {
            res = QFileDialog::getExistingDirectory(QString::null, 0, 0, caption, TRUE);
            break;
        }
        default:
        {
            Tcl_Obj* result;
            QCString utf = QObject::tr("Type must be one of following: %1").arg(types.join(", ")).utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
        }
    }

    if (res != QString::null)
    {
        Tcl_Obj* result;
        QCString utf = res.utf8();
        result = Tcl_NewStringObj((const char* const)utf, utf.length());
        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
    } else {
        Tcl_SetResult(interp, "", TCL_STATIC);
    }

    return TCL_OK;
}

/* Tcl Command user
GLOBALSYNTAX: user komenda ?argumenty?
DESCRIPTION:
Komenda suca zarzdzaniu list kontaktw.

SUBSYNTAX: user add ?-uin liczba -altname nazwa ?-firstname nazwa? ?-lastname nazwa? ?-nickname nazwa? ?-mobile numer? ?-blocking true/false? ?-offline true/false? ?-notify true/false? ?-group grupa? ?-email adres? ?-anonymous true/false??
DESCRIPTION:
Jeli adna z opcji nie jest podana, otwiera dialog dodawania nowego uytkownika. Jeli jakakolwiek opcja
jest podana, dodaje nowego uytkownika z podanymi, lub domylnymi parametrami. W tym przypadku rwnierz
<u>musz</u> by zdefiniowane 2 parametry: UIN (-uin) oraz nazwa alternatywna (-altname).<br><br>
<I>Nazwa alternatywna</I> (-altname) jest nazw wywietlan w oknie kontaktw.<br><br>
<I>UIN</I> (-uin) jest unikalnym identyfikatorem uytkownika GG.<br><br>
<I>Pseudonim</I> (-nickname) jest nazw zwykle identyczny z nazw alternatywn, lecz czasem
istnieje potrzeba zmiany nazwy wywietlanej na inn ni pseudonim, a mimo to, bdzie on pamitany
jako osobny parametr. Domylna warto to pusty acuch.<br><br>
<I>Blokowanie</I> (-blocking) wcza lub wycza blokowanie uytkownika. Domylna warto to FALSE.
Argument dla tej opcji jest dozwolony w formie TRUE/FALSE, jak rwnie 1/0.<br><br>
<I>Bd niedostpny</I> (-offline) dla tego uytkownika. Argument identyczny jak dla -blocking.
Domylnie jest FALSE.<br><br>
<I>Powiadamiaj o</I> (-notify) dostpnoci tego uytkownika. Argument identyczny jak dla -blocking.
Domylnie jest TRUE.<br><br>
<I>Anonimowy</I> (-anonymous) definiuje uytkownika tak, jak robi to Kadu automatycznie z uytkownikiem,
ktorego nie mamy na licie, a wysa on do nas jak wiadomo. Domylnie jest FALSE. Opcja ta musi wystpowa
razem z opcj -uin, inne opcje s wtedy ignorowane.<br><br>
<I>Grupa</I> (-group) do ktrej uytkownik ma przynalee. Domylnie jest to "Wszyscy".<br><br>
<I>Imi</I> (-firstname), <I>Nazwisko</I> (-lastname), <I>Numer telefonu</I> (-mobile),
oraz <I>E-Mail</I> (-email) s dobrze znane i nie wymagaj omawiania.<br>

SUBSYNTAX: user remove nazwaAlternatywna
DESCRIPTION:
Uswa uytkownika z listy, ktry jest znany pod podan <i>nazw alternatywn</i>. Jeli nie ma takiego uytkownika
na licie, po prostu nic si nie dzieje.

SUBSYNTAX: user list
DESCRIPTION:
Zwraca list nazw alternatywnych wszystkich kontaktw, czyli tych widzianych w oknie kontaktw.

SUBSYNTAX: user uins
DESCRIPTION:
Zwraca list numerw identyfikacyjnych (UIN'w) wszystkich kontaktw.

SUBSYNTAX: user change staraNazwaAlternatywna ?-firstname nazwa? ?-lastname nazwa? ?-nickname nazwa? ?-altname nazwa? ?-mobile numer? ?-uin numer? ?-blocking true/false? ?-offline true/false? ?-notify true/false? ?-group grupa? ?-email adres?
DESCRIPTION:
Zmienia parametry zarejestrowanego uytkownika, skojaonego z nazw alternatywn, podan jako staraNazwaAlternatywna.
Jeli uytkownik o podanej nazwie alternatywnej nie jest zarejestrowany, zwracany jest bd. Wszystkie opcje tutaj spotkane
s opisane w komendzie user add.

SUBSYNTAX: user get UIN/nazwaAlternatywna opcja
DESCRIPTION:
Zwraca warto parametru okrelonego <i>opcj</i> dla uytkownika o podanej <i>nazwie alternatywnej</i>.
Opcja moe by ktorkolwiek z dozwolonych dla komendy user add, oraz dodatkowo mona tutaj uywa
dodatkowo nastpujcych opcji:<br>
<b>-status</b> - zwraca aktualny status uytkownika<Br>
<b>-description</b> - zwraca opis danego kontaktu (o ile takowy ma aktualnie ustawiony).<br>
<b>-dns</b> - zwraca adres DNS (o ile jest ju gotowy [patrz zdarzenie <a href="on.html"><b>usldns</b></a>])<br>
<b>-ip</b> - zwraca adres IP<br>
<b>-port</b> - zwraca port z ktrego korzysta rozmwca ( = 1, gdy kontakt jest za NAT'em [maskarad])<br>
<b>-protocol</b> - zwraca wersj protokou uywanego przez rozmwc

SUBSYNTAX: user exists UIN/nazwaAlternatywna
DESCRIPTION:
Jeli uytkownik o podanym identyfikatorze <i>UIN</i> lub podanej <i>nazwie alternatywnej</i> jest zarejestrowany
to zwraca 1, w przeciwnym wypadku zwraca 0.
*/
int Tcl_CmdUser(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    QString mainsyntax = "user command ?arguments?";
    QString addsyntax = "user add ?-uin number -altname name ?-firstname name? ?-lastname name?\n     ?-nickname name? ?-mobile number? ?-blocking true/false?\n     ?-offline true/false? ?-notify true/false? ?-group group?     ?-email address? ?-anonymous true/false??";
    QString remsyntax = "user remove altName";
    QString chsyntax = "user change oldAltName ?-firstname name? ?-lastname name? ?-nickname name?\n     ?-altname name? ?-mobile number? ?-uin number? ?-blocking true/false?\n     ?-offline true/false? ?-notify true/false? ?-group group? ?-email address?";
    QString exsyntax = "user exists UIN/altName";
    QString getsyntax = "user get UIN/altName option";
    QString getsyntax_opts = QObject::tr("option should be: -uin, -nickname, -altname, -firstname, -lastname, -mobile, -blocking, -offline, -notify, -group, -email, -anonymous");

    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, mainsyntax.local8Bit());
        return TCL_ERROR;
    }
    bool rest, conv;
    QStringList cmds = QStringList::split(" ", "add remove list uins change exists get"),
                *tclUOpt = new QStringList(QStringList::split(" ", "-uin -altname -firstname -lastname -nickname -mobile -status -blocking -offline -notify -group -description -email -anonymous -dns -ip -port -protocol"));
    QString cmd = argv[1], uin, altNick, anon, str, arg, oldAltNick, opt;
    UserListElement *nu, user, _nu;
    int id;
    uint uint_uin;
    Tcl_Obj* result;
    QCString utf;

    switch (cmds.findIndex(cmd))
    {
        case 0:
        {
            if (argc > 2)
            {
                rest = argc % 2;
                if (rest)
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
                    return TCL_ERROR;
                }
                else
                {
                    nu = new UserListElement();
                    int id;
                    for (int cnt = 2; argc > cnt; cnt++)
                    {
                        str = argv[cnt];
                        cnt++;
                        arg = QString::fromUtf8(argv[cnt]);
                        if (str.startsWith("-"))
                        {
                            id = tclUOpt->findIndex(str);
                            switch (id)
                            {
                                case 0:
                                    nu->uin = arg.toInt();
                                    uin = arg;
                                    break;
                                case 1:
                                    nu->altnick = arg;
                                    altNick = arg;
                                    break;
                                case 2:
                                    nu->first_name = arg;
                                    break;
                                case 3:
                                    nu->last_name = arg;
                                    break;
                                case 4:
                                    nu->nickname = arg;
                                    break;
                                case 5:
                                    nu->mobile = arg;
                                    break;
                                case 7:
                                    nu->blocking = arg;
                                    break;
                                case 8:
                                    nu->offline_to_user = arg;
                                    break;
                                case 9:
                                    nu->notify = arg;
                                    break;
                                case 10:
                                    nu->setGroup(arg);
                                    break;
                                case 12:
                                    nu->email = arg;
                                    break;
                                case 13:
                                    nu->anonymous = arg;
                                    anon = arg;
                                    break;
                                default:
                                    Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
                                    return TCL_ERROR;
                            }
                        }
                        else
                        {
                            Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
                            return TCL_ERROR;
                        }
                    }
                    if (!uin.isNull())
                    {
                        if (!altNick.isNull())
                        {
                            userlist.addUser(*nu);
                        }
                        else if (!anon.isNull())
                        {
                            userlist.addAnonymous(uin.toInt());
                        }
                        else
                        {
                            Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
                            return TCL_ERROR;
                        }
                    }
                    else
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
                        return TCL_ERROR;
                    }
                }
                delete nu;
            }
            else
            {
                kadu->addUserAction();
            }
            break;
        }
        case 1:
        {
            if (argc == 3)
            {
                altNick = argv[2];
                if (userlist.containsAltNick(altNick))
                {
                    str = QString(QObject::tr("User %1 isn't registered")).arg(altNick);
                    Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
                    userlist.removeUser(altNick);
                }
                else
                    return TCL_ERROR;
            }
            else
                Tcl_WrongNumArgs(interp, 0, NULL, remsyntax.local8Bit());

            break;
        }
        case 2:
        {
            for (uint cnt = 0; userlist.count() > cnt; cnt++)
            {
                user = userlist[cnt];
                str = user.altnick;
                Tcl_Obj* result;
                QCString utf = str.utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_AppendElement(interp, Tcl_GetString(result));
            }
            break;
        }
        case 3:
        {
            for (uint cnt = 0; userlist.count() > cnt; cnt++)
            {
                user = userlist[cnt];
                Tcl_AppendElement(interp, (char*)QString::number(user.uin).latin1());
            }
            break;
        }
        case 4:
        {
            if (argc > 2)
            {
                rest = argc % 2;
                if (!rest)
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, chsyntax.local8Bit());
                    return TCL_ERROR;
                }
                else
                {
                    oldAltNick = argv[2];
                    if (userlist.containsAltNick(oldAltNick))
                    {
                        _nu = userlist.byAltNick(oldAltNick);
                        for (int cnt = 3; argc > cnt; cnt++)
                        {
                            str = argv[cnt];
                            cnt++;
                            arg = QString::fromUtf8(argv[cnt]);
                            if (str.startsWith("-"))
                            {
                                id = tclUOpt->findIndex(str);
                                switch (id)
                                {
                                    case 0:
                                        _nu.uin = arg.toInt();
                                        break;
                                    case 1:
                                        _nu.altnick = arg;
                                        break;
                                    case 2:
                                        _nu.first_name = arg;
                                        break;
                                    case 3:
                                        _nu.last_name = arg;
                                        break;
                                    case 4:
                                        _nu.nickname = arg;
                                        break;
                                    case 5:
                                        _nu.mobile = arg;
                                        break;
                                    case 7:
                                        _nu.blocking = arg;
                                        break;
                                    case 8:
                                        _nu.offline_to_user = arg;
                                        break;
                                    case 9:
                                        _nu.notify = arg;
                                        break;
                                    case 10:
                                        _nu.setGroup(arg);
                                        break;
                                    case 12:
                                        _nu.email = arg;
                                        break;

                                    default:
                                        Tcl_WrongNumArgs(interp, 0, NULL, addsyntax.local8Bit());
                                        return TCL_ERROR;
                                }
                            }
                            else
                            {
                                Tcl_WrongNumArgs(interp, 0, NULL, chsyntax.local8Bit());
                                return TCL_ERROR;
                            }
                        }
                        userlist.changeUserInfo(oldAltNick, _nu);
                    }
                    else
                    {
                        str = QString(QObject::tr("User isn't registered: %1\nwhile executing:\nuser change %1")).arg(oldAltNick);
                        Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                }
            }
            else
            {
                Tcl_WrongNumArgs(interp, 0, NULL, chsyntax.local8Bit());
                return TCL_ERROR;
            }
            break;
        }
        case 5:
        {
            if (argc == 3)
            {
                altNick = QString::fromUtf8(argv[2]);
                uint_uin = altNick.toUInt(&conv);
                if (conv == TRUE)
                {
                    if (userlist.containsUin(uint_uin))
                    {
                        Tcl_SetResult(interp, "1", TCL_STATIC);
                    }
                    else
                    {
                        if (userlist.containsAltNick(altNick))
                        {
                            Tcl_SetResult(interp, "1", TCL_STATIC);
                        }
                        else
                        {
                            Tcl_SetResult(interp, "0", TCL_STATIC);
                        }
                    }
                }
                else
                {
                    if (userlist.containsAltNick(altNick))
                    {
                        Tcl_SetResult(interp, "1", TCL_STATIC);
                    }
                    else
                    {
                        Tcl_SetResult(interp, "0", TCL_STATIC);
                    }
                }
            }
            else
            {
                Tcl_WrongNumArgs(interp, 0, NULL, exsyntax.local8Bit());
                return TCL_ERROR;
            }
            break;
        }
        case 6:
        {
            if (argc == 4)
            {
                altNick = QString::fromUtf8(argv[2]);
                opt = argv[3];
                uint_uin = altNick.toUInt(&conv);

                bool searchingForAnon = false, foundUser = true;
                if (opt.startsWith("-") && tclUOpt->findIndex(opt) == 13 )
                        searchingForAnon = true;
                
                if (conv == TRUE)
                {
                    if (userlist.containsUin(uint_uin))
                    {
                        user = userlist.byUin(uint_uin);
                    }
                    else
                    {
                        if (userlist.containsAltNick(altNick))
                        {
                            user = userlist.byAltNick(altNick);
                        }
                        else
                        {
                            if ( searchingForAnon )
                                foundUser = false;
                            else
                            {
                                str = QString("User UIN isn't registered: %1\nwhile executing:\nuser get %2 %3").arg(altNick).arg(altNick).arg(opt);
                                Tcl_SetResult(interp, (char *)str.latin1(), TCL_VOLATILE);
                                return TCL_ERROR;
                            }
                        }
                    }
                }
                else
                {
                    if (userlist.containsAltNick(altNick))
                    {
                        user = userlist.byAltNick(altNick);
                    }
                    else
                    {
                        if ( searchingForAnon )
                        {
                              foundUser = false;
                        }
                        else
                        {
                            str = QString(("User UIN isn't registered: %1\nwhile executing:\nuser get %1 %2")).arg(altNick).arg(opt);
                            Tcl_SetResult(interp, (char*)str.latin1(), TCL_VOLATILE);
                            return TCL_ERROR;
                        }
                     }
                }
                if (opt.startsWith("-"))
                {
                    id = tclUOpt->findIndex(opt);
                    switch (id)
                    {
                        case 0:
                            arg = QString::number(user.uin);
                            break;
                        case 1:
                            arg = user.altnick;
                            break;
                        case 2:
                            arg = user.first_name;
                            break;
                        case 3:
                            arg = user.last_name;
                            break;
                        case 4:
                            arg = user.nickname;
                            break;
                        case 5:
                            arg = user.mobile;
                            break;
                        case 6:
                            arg = TclMisc::GGStatusToTclStatus(user.status);
                            if (arg.isEmpty())
                            {
                                utf = QObject::tr("Unknown status type.").utf8();
                                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                                return TCL_ERROR;
                            }
                            break;
                        case 7:
                            arg = QString::number(user.blocking);
                            break;
                        case 8:
                            arg = user.offline_to_user;
                            break;
                        case 9:
                            arg = QString::number(user.notify);
                            break;
                        case 10:
                            arg = user.group();
                            break;
                        case 11:
                            arg = user.description;
                            break;
                        case 12:
                            arg = user.email;
                            break;
                        case 13:
                            if ( foundUser )
                                foundUser = user.anonymous;

                            arg = QString::number(foundUser);
                            break;
                        case 14:
                            arg = user.dnsname;
                            break;
                        case 15:
                            arg = user.ip.toString();
                            break;
                        case 16:
                            arg = QString::number(user.port);
                            break;
                        case 17:
                            arg = QString::number(user.version);
                            break;
                        default:
                            utf = getsyntax_opts.utf8();
                            result = Tcl_NewStringObj((const char* const)utf, utf.length());
                            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                            return TCL_ERROR;
                    }
                    utf = arg.utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                }
                else
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, getsyntax.local8Bit());
                     return TCL_ERROR;
                }
            }
            else
            {
                Tcl_WrongNumArgs(interp, 0, NULL, getsyntax.local8Bit());
                return TCL_ERROR;
            }
            break;
        }
        default:
            str = QString(QObject::tr("Command should be one of following: %1")).arg(cmds.join(", "));
            Tcl_WrongNumArgs(interp, 0, NULL, str.local8Bit());
            return TCL_ERROR;
    }
    delete tclUOpt;
    return TCL_OK;
}

/* Tcl Command ignore
GLOBALSYNTAX: ignore komenda ?argumenty?
DESCRIPTION:
Komenda suca manipulowaniu list ignorowanych.

SUBSYNTAX: ignore add ?UIN ?UIN ?...??? ?lista UIN'w ?lista UIN'w ?...???
DESCRIPTION:
Dodaje podane UIN'y, lub listy UIN'w do listy ignorowanych.

SUBSYNTAX: ignore remove ?UIN ?UIN ?...??? ?lista UIN'w ?lista UIN'w ?...???
DESCRIPTION:
Usuwa podane UIN'y, lub listy UIN'w z listy ignorowanych.

SUBSYNTAX: ignore exists ?UIN ?UIN ?...??? ?lista UIN'w ?lista UIN'w ?...???
DESCRIPTION:
Sprawdza czy podany UIN, lub lista UIN'w jest dodana do listy ignorowanych. Jeli tak, zwraca 1, inaczej zwraca 0.

SUBSYNTAX: ignore list
DESCRIPTION:
Zwraca liste pozycji ignorowanych, w ktrej mog si znale pojedyncze UIN'y, jak i listy UIN'w.
*/
int Tcl_CmdIgnore(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "ignore command ?arguments?");
        return TCL_ERROR;
    }
    QString cmd = argv[1], arg;
    QStringList cmds = QStringList::split(" ", "add remove exists list"), args;
    UinsList igList, ig_List, emptyList, uins;
      bool conv;
    uint uin;
    uint i, j;
    Tcl_DString ds;

    switch (cmds.findIndex(cmd))
    {
        case 0:
        {
            for (int cnt = 2; cnt < argc; cnt++)
            {
                arg = argv[cnt];

                // Sprawdzanie, czy dany argument to lista UIN'ow czy pojedynczy UIN
                if (arg.find(" ") > -1)
                {
                    args = QStringList::split(" ", arg);
                    ig_List = emptyList;

                    // Dla kazdego UIN'a z listy sprawdzanie, czy jest poprawnym UIN'em.
                    for ( QStringList::Iterator it = args.begin(); it != args.end(); ++it )
                    {
                        uin = (*it).toUInt(&conv);
                        if (conv)
                        {
                            if (!ig_List.findIndex(uin) > -1)
                            {
                                // Dodawanie UIN'a do listy, aby sie nie powtorzyl
                                ig_List.append(uin);
                            }
                        }
                        else
                        {
                            Tcl_WrongNumArgs(interp, 0, NULL, "ignore add ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
                            return TCL_ERROR;
                        }
                    }
                    if (!isIgnored(ig_List))
                        addIgnored(ig_List);

                }
                else
                {

                    // Sprawdzanie czy podany UIN jest poprawny.
                    uin = arg.toUInt(&conv);
                    if (conv)
                    {
                        if (!igList.findIndex(uin) > -1)
                        {
                            // Deklarowanie uins, aby dodawany UIN byl lista
                            uins.append(uin);
                            if (!isIgnored(uins))
                                addIgnored(uins);
                        }
                    }
                    else
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, "ignore add ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
                        return TCL_ERROR;
                    }
                }
            }
            break;
        }
        case 1:
        {
            for (int cnt = 2; cnt < argc; cnt++)
            {
                arg = argv[cnt];

                // Sprawdzanie, czy dany argument to lista UIN'ow czy pojedynczy UIN
                if (arg.find(" ") > -1)
                {
                    args = QStringList::split(" ", arg);
                    ig_List = emptyList;

                    // Dla kazdego UIN'a z listy sprawdzanie, czy jest poprawnym UIN'em.
                    for ( QStringList::Iterator it = args.begin(); it != args.end(); ++it )
                    {
                        uin = (*it).toUInt(&conv);
                        if (conv)
                        {
                            if (!ig_List.findIndex(uin) > -1)
                            {
                                // Dodawanie UIN'a do listy, aby sie nie powtorzyl
                                ig_List.append(uin);
                            }
                        }
                        else
                        {
                            Tcl_WrongNumArgs(interp, 0, NULL, "ignore add ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
                            return TCL_ERROR;
                        }
                    }
                    if (ignored.findIndex(ig_List) > -1)
                        delIgnored(ig_List);

                }
                else
                {
                    uin = arg.toUInt(&conv);
                    if (conv)
                    {
                        uins.append(uin);
                        if (ignored.findIndex(uins) > -1)
                            delIgnored(uins);

                    }
                    else
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, "ignore remove ?UIN ?UIN ?...??? ?UINs list ?UINs list ?...???");
                        return TCL_ERROR;
                    }
                }
            }
            break;
        }
        case 2:
        {
            if (argc > 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "ignore exists ?UIN? ?UINs list?");
                return TCL_ERROR;
            }
            else
            {
                arg = argv[2];
                uin = arg.toUInt(&conv);
                if (!conv)
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, "ignore exists ?UIN? ?UINs list?");
                    return TCL_ERROR;
                }
                uins.append(uin);
                if (ignored.findIndex(uins) > -1)
                {
                    Tcl_SetResult(interp, "1", TCL_STATIC);
                }
                else
                {
                    Tcl_SetResult(interp, "0", TCL_STATIC);
                }
            }
            break;
        }
        case 3:
        {
            Tcl_DStringInit(&ds);
            for (i = 0; i < ignored.count(); i++)
            {
                Tcl_DStringStartSublist(&ds);
                for (j = 0; j < ignored[i].count(); j++)
                    Tcl_DStringAppendElement(&ds, (char*)QString::number(ignored[i][j]).latin1() );

                Tcl_DStringEndSublist(&ds);
            }
            Tcl_DStringResult(interp, &ds);
            Tcl_DStringFree(&ds);
            break;
        }
        default:
            arg = QString(QObject::tr("Command should be one of following: %1")).arg(cmds.join(", "));
            Tcl_WrongNumArgs(interp, 0, NULL, arg.local8Bit());
            return TCL_ERROR;
    }
    return TCL_OK;
}

/* Tcl Command play
SYNTAX: play ?-force? plik
DESCRIPTION:
Odtwarza plik muzyczny. Jeli plik nie istnieje, nic nie zostanie odtworzone.
Jeli zostanie uyta opcja <b>-force</b>, to dwik zostanie odtworzony
bez wzgldu na to, czy dwieki w Kadu s wyczone, czy nie.
*/
int Tcl_CmdPlay(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 3)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "play ?-force? file");
        return TCL_ERROR;
    }
    bool force = FALSE;
    int arg;
    if (argc == 3)
    {
        if (!QString::compare(argv[1], "-force"))
            force = TRUE;
        else
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "play ?-force? file");
            return TCL_ERROR;
        }
        arg = 2;
    }
    else
        arg = 1;

    QString file = QString::fromUtf8(argv[arg]);
    if (sound_manager == NULL)
    {
        QCString utf = QObject::tr("Sound module isn't loaded!").utf8();
        Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
        return TCL_ERROR;
    }

    sound_manager->play(file, force);
    return TCL_OK;
}

/* Tcl Command configfile
GLOBALSYNTAX: configfile komenda ?argumenty?
DESCRIPTION:
Suy do odczytu i zapisu zmiennych w pliku konfiguracyjnym Kadu.

SUBSYNTAX: configfile read grupa zmienna
DESCRIPTION:
Czyta z pliku konfiguracyjnego, spod podanej <i>grupy</i> ustawie, dan <i>zmienn</i>.

SUBSYNTAX: configfile write grupa zmienna warto
DESCRIPTION:
Zapisuje w pliku konfiguracyjnym <i>zmienn</i>, pod podan <i>grup</i>,
przyporzdkowyjc jej dan warto.
*/
int Tcl_CmdConfigFile(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "configfile command arguments");
        return TCL_ERROR;
    }
    QStringList cmds = QStringList::split(" ", "read write");
    QString act = argv[1], group, var, res, value;
    Tcl_Obj* result;
    QCString utf;

    switch (cmds.findIndex(act))
    {
        case 0:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "configfile read group variable");
                return TCL_ERROR;
            }
            group = QString::fromUtf8(argv[2]);
            var = QString::fromUtf8(argv[3]);
            res = config_file.readEntry(group, var);
            utf = res.utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            break;
        }
        case 1:
        {
            if (argc != 5)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "configfile write group variable value");
                return TCL_ERROR;
            }
            group = QString::fromUtf8(argv[2]);
            var = QString::fromUtf8(argv[3]);
            value = QString::fromUtf8(argv[4]);
            config_file.writeEntry(group, var, value);
            break;
        }
        default:
            res = QString(QObject::tr("Command should be one of following: %1")).arg(cmds.join(", "));
            Tcl_WrongNumArgs(interp, 0, NULL, res.local8Bit());
            return TCL_ERROR;
    }
    return TCL_OK;
}

/* Tcl Command chat
GLOBALSYNTAX: chat komenda ?argumenty?
DESCRIPTION:
Komenda chat suy do operacji na oknach rozmw.

SUBSYNTAX: chat open UIN/UINy
DESCRIPTION:
Otwiera okno rozmowy/konferencij dla podanego <i>UINu</i> lub <i>UINw</i>. Jeli okno istnieje, zwracany
jest bd.

SUBSYNTAX: chat close UIN/UINy
DESCRIPTION:
Zamyka okno rozmowy/konferencji z podanym <i>UINem</i> lub <i>UINami</i>. Jeli okno nie istnieje, zwracany
jest bd.

SUBSYNTAX: chat exists UIN/UINy
<dd><br><p>
DESCRIPTION:
Zwraca 1, jeli okno rozmowy/konferencji z podanym <i>UINem</i> lub <i>UINami</i> istnieje, w przeciwnym wypadku
zwraca 0.

SUBSYNTAX: chat input UIN/UINy ?-html? ?warto?
DESCRIPTION:
Jeli podana jest warto, to pole wiadomoci w oknie rozmowy z podanym
<i>UINem</i>, bd <i>list UINw</i> (konferencja) bdzie t warto zawierao. Jeli zostanie
ona opuszczona, to aktualna zawarto tego pola zostanie zwrcona. Jeli okno rozmowy z podanym <i>UINem</i>,
czy <i>list UINw</i> nie istnieje, zostanie zwrcony bd. Jeli nie ma pewnoci, e okno istnieje, naley
posuy si komend chat exists. Gdy podana zostanie opcja -html, to w przypadku zwracania zawartoci
pola rozmowy, zwrcone te zostan tagi HTML (jak pogrubienie, etc.), a w przypadku nadawania nowej wartoci
temu polu, wszelkie tagi HTML zostan zinterpretowane, zamiast wstawione, jako zwyky tekst.

SUBSYNTAX: chat addtext UIN/UINy ?-formated? ?-addhistory? tekst
DESCRIPTION:
Dodaje podany <i>tekst</i> do okna rozmowy z <i>UINem</i>, a dokladniej
do listy wywietlanych wiadomoci. Jeli uyty zostanie przecznik -formated,
to tekst zostanie wywietlony jako otrzymana wiadomo od <i>UINu</i>, w przeciwnym
wypadku tekst zostanie wywietlony dokadnie w takiej formie, w jakiej zostanie podany.
Przecznik -addhistory spowoduje zapisanie dodawanej wiadomoci w historii rozmowy z podanymi
<i>UINami</i>. Mona w nim uywa znacznikw HTML.

SUBSYNTAX: chat clear UIN/UINy
DESCRIPTION:
Czyci wszystkie wiadomoci z okna rozmowy z <i>UINem</i>.

SUBSYNTAX: chat list
DESCRIPTION:
Zwraca list UINw, z ktrymi otwarte s okna rozmw.

SUBSYNTAX: chat curpos UIN/UINy ?pozX pozY?
DESCRIPTION:
Jeli podane s <i>pozycje X i Y</i>, to ustawia kursor w oknie rozmowy z <i>UINem</i> bd
<i>UINami</i> na podan pozycj. Jeli natomiast argumenty te nie s podane, zwraca aktualn
pozycj kursora, w postaci listy 2 wyrazw.

SUBSYNTAX: chat selection UIN/UINy ?startPozX startPozY koniecPozX koniecPozY?
DESCRIPTION:
Jeli podane s <i>pozycje pocztkowe X i Y</i> oraz <i>kocowe X i Y</i> to zaznaczony zostaje
tekst w oknie rozmowy z <i>UINem</i> bd <i>UINami</i>, poczwszy od pozycji okrelonej dwoma
pierwszymi argumentami, a skonczywszy na pozycji okrelonej ostatnimi dwoma. Jeli <i>pozycje</i>
nie zostan podane, zwracana jest 4-wyrazowa lista zawierajca ww. pozycje dla bierzcego tekstu
zaznaczonego. Jeli aden tekst nie jest zaznaczony, wszystkie zwrcone liczby bda mie warto -1.

SUBSYNTAX: chat getselection UIN/UINy
DESCRIPTION:
Jeli jest zaznaczony jakikolwiek tekst w oknie rozmowy z <i>UINem</i> bd <i>UINami</i>,
to zostaje on zwrcony. W przeciwnym wypadku pusty acuch jest zwracany.

SUBSYNTAX: chat raise UIN/UINy
DESCRIPTION:
Przywouje otwarte oknorozmowy  na pierwszy plan, oraz sprawia e tytu okna miga si, a do momentu,
a uytkownik kliknie na tym oknie. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat minimize UIN/UINy
DESCRIPTION:
Minimalizuje okno rozmowy. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat restore UIN/UINy
DESCRIPTION:
Przywraca okno rozmowy uprzednio zminimalizowane. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat maximize UIN/UINy
DESCRIPTION:
Maksymalizuje rozmiar okna rozmowy. Jeli okno nie istnieje, zwraca bd.

SUBSYNTAX: chat fixemotspath
DESCRIPTION:
Zwraca sciezke wskazujaca na katalog glowny /. Komenda ta jest potrzebna, gdy chcemy doda jaki
obrazek przy pomocy komendy <b>chat addtext</b>, jako e uycie tam <font face=courier>
&lt;img src="..."&gt;</font> spowoduje prb odczytu podanego obrazka spod cieki
/katalog/instalacji/kadu/share/themes/emoticons/styl-emotikon/. Komenda <b>chat fixemotspath</b>
zwraca odpowiedni ancuch <font face=courier>../../..</font> (powtrzone odpowiedni liczb razy),
ktry uyty jako pocztek cieki dla obrazka, umoliwia podanie cieki do obrazka od katalogu
gwnego.

EXAMPLE:
chat addtext 35234332 "&lt;img src=\"[chat fixemotspath]/home/user/.gg/blablabla.png\"&gt;&lt;br&gt;"
ENDOFEXAMPLE
*/
int Tcl_CmdChat(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "chat command ?arguments?");
        return TCL_ERROR;
    }
    QStringList acts = QStringList::split(" ", "input open close exists list curpos selection addtext raise minimize restore maximize fixemotspath clear getselection"), lst;
    QString act = argv[1], uin;
    UinsList uins;
    uint i, j;
    int x, y, x2, y2;
    QCString _utf = QObject::tr("No such chat window.").utf8();
    Tcl_Obj* _result = Tcl_NewStringObj((const char* const)_utf, _utf.length());

    switch (acts.findIndex(act))
    {
        case 0:
        {
            if (argc < 3 || argc > 6)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat input UIN/UINsList ?-html? ?value?");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                bool put = FALSE, html = FALSE;
                int textArg = 3;
                if (argc > 3)
                {
                    if (argc == 5)
                    {
                        if (QString::compare(argv[3], "-html"))
                        {
                            Tcl_WrongNumArgs(interp, 0, NULL, "chat input UIN/UINsList ?-html? ?value?");
                            return TCL_ERROR;
                        }
                        html = TRUE;
                        put = TRUE;
                        textArg++;
                    }
                    else
                    {
                        if (!QString::compare(argv[3], "-html"))
                            html = TRUE;
                        else
                            put = TRUE;
                    }
                }

                if (put)
                {
                    HtmlDocument doc, putDoc;
                    Chat* c = chat_manager->findChatByUins(uins);
                    QString start, end;

                    doc.parseHtml(c->edit->text());
                    int i, tostop = doc.countElements() - 5;
                    if (doc.countElements() > 0)
                    {
                        for ( i = 0; i < 7; i++ )
                        {
                            start.append(doc.elementText(i));
                        }
                        for ( i = tostop; i < doc.countElements(); i++ )
                        {
                            end.append(doc.elementText(i));
                        }
                        if (html)
                        {
                            start.append(QString::fromUtf8(argv[textArg]));
                            start.append(end);
                            putDoc.parseHtml(start);
                        }
                        else
                        {
                            putDoc.addTag(start);
                            putDoc.addText(QString::fromUtf8(argv[textArg]));
                            putDoc.addTag(end);
                        }
                    }
                    else
                    {
                        if (html)
                            putDoc.parseHtml(QString::fromUtf8(argv[textArg]));
                        else 
                            putDoc.addText(QString::fromUtf8(argv[textArg]));
                    }
                    chat_manager->findChatByUins(uins)->edit->setText(putDoc.generateHtml());
                }
                else
                {
                    HtmlDocument doc;
                    Chat* c = chat_manager->findChatByUins(uins);
                    QCString str;
                    Tcl_DString ds;
                    Tcl_DStringInit(&ds);
                    Tcl_Obj* result;

                    doc.parseHtml(c->edit->text());
                    int stop = doc.countElements() - 5;
                    if (html)
                    {
                        for ( int i = 7; i < stop; i++ )
                        {
                            str = doc.elementText(i).utf8();
                            result = Tcl_NewStringObj((const char* const)str, str.length());
                            Tcl_DStringAppend(&ds, Tcl_GetString(result), str.length());
                        }
                    }
                    else
                    {
                        for ( int i = 7; i < stop; i++ )
                        {
                            if (!doc.isTagElement(i))
                            {
                                str = doc.elementText(i).utf8();
                                result = Tcl_NewStringObj((const char* const)str, str.length());
                                Tcl_DStringAppend(&ds, Tcl_GetString(result), str.length());
                            }
                        }
                    }
                    Tcl_DStringResult(interp, &ds);
                    Tcl_DStringFree(&ds);
                }
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 1:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat open UIN/UINsList");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                QCString utf = QObject::tr("Chat window already exists.").utf8();
                Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            else
            {
                chat_manager->openChat(uins, QDateTime::currentDateTime().toTime_t());
            }
            break;
        }
        case 2:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat close UIN/UINsList");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                chat_manager->findChatByUins(uins)->close(TRUE);
            }
            else
            {
                QCString utf = QObject::tr("Chat window doesn't exist.").utf8();
                Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 3:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat exists UIN/UINsList");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                Tcl_SetResult(interp, "1", TCL_STATIC);
            }
            else
            {
                Tcl_SetResult(interp, "0", TCL_STATIC);
            }
            break;
        }
        case 4:
        {
            if (argc != 2)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat list");
                return TCL_ERROR;
            }
            for ( i = 0; chat_manager->chats().count() > i; i++ )
            {
                lst.clear();
                for ( j = 0; chat_manager->chats()[i]->uins().count() > j; j++ )
                {
                    lst.append(QString::number( chat_manager->chats()[i]->uins()[j] ));
                }
                Tcl_AppendElement(interp, lst.join(" ").local8Bit());
            }
            break;
        }
        case 5:
        {
            if (argc != 3 && argc != 5)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat curpos UIN/UINsList ?posX posY?");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                QString tmp;
                if (argc == 5)
                {
                    x = atoi(argv[3]);
                    y = atoi(argv[4]);
                    chat_manager->findChatByUins(uins)->edit->setCursorPosition(y, x);
                }
                else
                {
                    chat_manager->findChatByUins(uins)->edit->getCursorPosition(&y, &x);
                    Tcl_AppendElement(interp, (char*)QString::number(x).latin1());
                    Tcl_AppendElement(interp, (char*)QString::number(y).latin1());
                }
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 6:
        {
            if (argc != 3 && argc != 7)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat selection UIN/UINsList ?startPosX startPosY endPosX endPosY?");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                QString tmp;
                if (argc == 7)
                {
                    x = atoi(argv[3]);
                    y = atoi(argv[4]);
                    x2 = atoi(argv[5]);
                    y2 = atoi(argv[6]);
                    chat_manager->findChatByUins(uins)->edit->setSelection(y, x, y2, x2);
                }
                else
                {
                    chat_manager->findChatByUins(uins)->edit->getSelection(&y, &x, &y2, &x2);
                    Tcl_AppendElement(interp, (char*)QString::number(x).latin1());
                    Tcl_AppendElement(interp, (char*)QString::number(y).latin1());
                    Tcl_AppendElement(interp, (char*)QString::number(x2).latin1());
                    Tcl_AppendElement(interp, (char*)QString::number(y2).latin1());
                }
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 7:
        {
            if (argc < 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat addtext UIN ?-formated sender? ?-addhistory? text");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                QString tmp, tmp2, sender;
                bool formated = FALSE, addhistory = FALSE;
                uint arg = 3, optc = argc-1;
                if (argc > 4)
                {
                    QStringList opts = QStringList::split(" ", "-formated -addhistory");
                    for ( uint i = 3; i < optc; i++ )
                    {
                        switch (opts.findIndex(argv[i]))
                        {
                            case 0:
                            {
                                formated = TRUE;
                                i++;
                                arg++;
                                if (i < optc)
                                {
                                    sender = QString::fromUtf8(argv[i]);
                                }
                                else
                                {
                                    Tcl_WrongNumArgs(interp, 0, NULL, "chat addtext UIN/UINy ?-formated sender? ?-addhistory? text");
                                    return TCL_ERROR;
                                }
                                break;
                            }
                            case 1:
                            {
                                addhistory = TRUE;
                                break;
                            }
                            default:
                                Tcl_WrongNumArgs(interp, 0, NULL, "chat addtext UIN/UINy ?-formated sender? ?-addhistory? text");
                                return TCL_ERROR;
                        }
                        arg++;
                    }
                }

                tmp = QString::fromUtf8(argv[arg]);

                if (addhistory)
                {
                    if (config_file.readBoolEntry("General","Logging"))
                        history.appendMessage(uins, uins[0], tmp, true);
                }

                if (formated)
                {
                    chat_manager->findChatByUins(uins)->formatMessage(true, sender, tmp, timestamp(), tmp2);
                    tmp = tmp2;
                }

                chat_manager->findChatByUins(uins)->scrollMessages(tmp);
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 8:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat raise UIN/UINy");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                chat_manager->findChatByUins(uins)->raise();
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 9:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat minimize UIN/UINy");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                chat_manager->findChatByUins(uins)->showMinimized();
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 10:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat restore UIN/UINy");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                chat_manager->findChatByUins(uins)->showNormal();
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 11:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat maximize UIN/UINy");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                chat_manager->findChatByUins(uins)->showMaximized();
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 12:
        {
            if (argc != 2)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat fixemotspath");
                return TCL_ERROR;
            }
            QString theme = emoticons->themePath(), tmp = "";
            for ( int i = 0; i < theme.contains("/"); i++ )
                tmp += "../";

            QCString str = tmp.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)str, str.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            break;
        }
        case 13:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat clear UIN/UINy");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
            {
                uins.append(uin.toUInt());
            }
            if (chat_manager->findChatByUins(uins))
            {
                chat_manager->findChatByUins(uins)->body->clear();
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 14:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "chat getselection UIN/UINsList");
                return TCL_ERROR;
            }
            uin = argv[2];
            if (uin.find(" ") > -1)
            {
                QStringList list = QStringList::split(" ", uin);
                for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
                {
                    uins.append((*it).toUInt());
                }
            }
            else
                uins.append(uin.toUInt());

            if (chat_manager->findChatByUins(uins))
            {
                QString str, tmp = chat_manager->findChatByUins(uins)->edit->selectedText();
                HtmlDocument doc;
                doc.parseHtml(tmp);
                for ( int i = 0; i < doc.countElements(); i++ )
                    if (!doc.isTagElement(i))
                        str = doc.elementText(i);
                
                QCString utf = tmp.utf8();
                Tcl_Obj* result = Tcl_NewStringObj((const char* const)str, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            }
            else
            {
                Tcl_SetResult(interp, Tcl_GetString(_result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        default:
        {
            QString tmp = QString(QObject::tr("Command should be one of following:\n%1")).arg(acts.join(", "));
            Tcl_WrongNumArgs(interp, 0, NULL, tmp.local8Bit());
            return TCL_ERROR;
        }
    }
    return TCL_OK;
}

/* Tcl Command hint
SYNTAX: hint tekst ?-icon ikona? ?-font czcionka? ?-fgcolor kolor? ?-bgcolor kolor? ?-timeout sekundy?

DESCRIPTION:
Wywietla tzw. dymki obok zadokowanej ikony Kadu. Jako pierwszy argument powinien by podany
tekst ktry bdzie stanowi informacj zawart w dymku. Moe on zawiera znaczniki HTML.
Pozostae argumenty s opcjonalne.<br><br>
<dt>
<P><DT><B>-icon</b><dd>
definiuje, jaka ikona ma si pojawi po lewej stronie informacji tekstowej. Domylnie pusta.<br>
<P><DT><b>-font</b><dd>
definiuje czcionk, ktra bdzie uyta w dymku. Domylnie jest to helvetica, rozmiar 12.
Argument czcionki moe mie 2 formy: pojedyncze sowo - nazw czcionki, lub te dwa sowa
(jako jeden argument TCL! czyli w cudzysowiu lub nawiasach klamrowych), gdzie pierwsze
to nazwa czcionki, a drugie to jej rozmiar.<br>
<P><DT><b>-fgcolor</b><dd>
okrela kolor czcionki. Domylnie czarny (black).<br>
<P><DT><b>-bgcolor</b><dd>
okrela kolor ta. Domylnie szary (grey).<br>
<P><DT><b>-timeout</b><dd>
ustawia czas wywietlania dymka w sekundach. Domylnie 3 sekundy.<br>
<br>
<i>Kolory</i> mog by podane w postaci #RRGGBB lub jako anglo-jzycznej nazwy kolorw podstawowych, jak
black, white, green, grey, itd. Czcionka oznacza rodzaj uytej czcionki, ale mona te po spacji
(ale w tym samym argumencie!) poda jej rozmiar.
*/
int Tcl_CmdHint(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2 || (argc % 2))
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "hint text ?-icon icon? ?-font font? ?-fgcolor color? ?-bgcolor color? ?-timeout secs?");
        return TCL_ERROR;
    }
    QString msg = QString::fromUtf8(argv[1]), tmp;
    QPixmap icon;
    QFont font = QString("helvetica");
    QColor fg = QString("black"), bg = QString("grey");
    QStringList fontList;
    int timeout = 3, size = 12;
    bool ok;

    QStringList opts = QStringList::split(" ", "-icon -font -fgcolor -bgcolor -timeout");
    for ( int c = 2; c < argc; c++ )
    {
        tmp = argv[c];
        c++;
        switch (opts.findIndex(tmp))
        {
            case 0:
            {
                icon = icons_manager.loadIcon(argv[c]);
                break;
            }
            case 1:
            {
                fontList = QStringList::split(" ", argv[c]);
                if (fontList.count() > 1)
                {
                    size = fontList[1].toUInt(&ok);
                    if (!ok)
                    {
                        QCString utf = QObject::tr("second argument of option -font should be integer value.").utf8();
                        Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                }
                font = QFont(fontList[0], size);
                break;
            }
            case 2:
            {
                fg = QString(argv[c]);
                break;
            }
            case 3:
            {
                bg = QString(argv[c]);
                break;
            }
            case 4:
            {
                timeout = QString(argv[c]).toUInt(&ok);
                if (!ok)
                {
                    QCString utf = QObject::tr("-timeout option accepts only integer values.").utf8();
                    Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
                break;
            }
            default:
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "hint text ?-icon icon? ?-font font? ?-fgcolor color? ?-bgcolor color? ?-timeout secs?");
                return TCL_ERROR;
            }
        }
    }
    hintmanager->addHint(msg, icon, font, fg, bg, timeout);
    return TCL_OK;
}

/* Tcl Command timer
SYNTAX: timer ?-repeat? milisekundy kod
SYNTAX: timer -delete ID
SYNTAX: timer list

DESCRIPTION:
Skadnia przypomina t najczciej uywan z komendy after. Wykonuje
podany kod po upywie podanej liczby <i>milisekund</i>. Zwraca ID (identyfikator)
dla danego licznika, ktrego pniej mona uy z opcj -delete, aby zatrzyma odliczanie
i usun licznik. timer list zwraca list licznikw, w ktrej kady element jest dwywurazowy
- pierwszy wyraz to ID licznika, a drugi to przyporzdkowany mu kod. Opcja -repeat
oznacza, e podany kod bdzie wykonywany co podan liczb <i>milisekund</i>, a do momentu
usunicia czasomierza, co pozwala na budowanie ptli niekoniecznie rekurencyjnych.
*/
int Tcl_CmdTimer(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 4 || argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "timer ?\?-repeat? msecs code? ?-delete id? ?list?");
        return TCL_ERROR;
    }
    int time, id, arg, sessID = *((int*) clientData);
    bool ok, ss = TRUE;
    QString cmd = argv[1];
    TclSession *tsess = tclSess[sessID];

    if (!cmd.compare("-delete"))
    {
        if (argc != 3)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "timer -delete id");
            return TCL_ERROR;
        }
        id = QString(argv[2]).toInt(&ok);
        if (!ok)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "timer -delete id");
            return TCL_ERROR;
        }

        uint i = 0;
        while ( i < tsess->tclTimers.count() && tsess->tclTimers[i].id != id )
            i++;

        if ( i == tsess->tclTimers.count() )
        {
            QCString utf = QObject::tr("Timer doesn't exists!").utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
        }

        tsess->tclTimers[i].ptr->kill();

    }
    else if (!cmd.compare("list"))
    {
        if (argc != 2)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "timer list");
            return TCL_ERROR;
        }
        for ( uint i = 0; i < tsess->tclTimers.count(); i++ )
            Tcl_AppendElement(interp, (char*)QString::number(tsess->tclTimers[i].id).latin1());

        return TCL_OK;

    }
    else
    {
        if (argc == 3)
        {
            arg = 2;
            time = cmd.toInt(&ok);
        }
        else if (argc == 4 && !cmd.compare("-repeat") )
        {
            arg = 3;
            cmd = argv[2];
            time = cmd.toInt(&ok);
            ss = FALSE;
        }
        else
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "timer ?-repeat? msecs code");
            return TCL_ERROR;
        }
        
        if (!ok)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "timer ?-repeat? msecs code");
            return TCL_ERROR;
        }

        TclTimer *timer = new TclTimer(tsess);
        timerID ID;

        timer->setCode(QString::fromUtf8(argv[arg]));

        // Rejestracja nowego ID
        timer->setSingleShot(ss);
        ID.ptr = timer;
        id = timer->start(time, ss);
        timer->currIdent(id);
        ID.id = id;
        tsess->tclTimers.append(ID);

        // Zwrocenie ID
        Tcl_SetResult(interp, (char*)QString::number(id).latin1(), TCL_VOLATILE);

    }

    return TCL_OK;
}

/* Tcl Command on
SYNTAX: on typ identyfikator ?procedura?
SYNTAX: on typ -delete identyfikator

DESCRIPTION:
Komenda on suy do obslugi zdarze. Sposobem uycia przypomina nieco
komend 'bind' z IRC bota - Eggdropa. Kademu zdarzeniu
moe towarzyszy jedno lub wicej wywoa procedur. Typ podawany
jako pierwszy z argumentw okrela rodzaj zdarzenia, ktre chcemy obsuy.
Identyfikator musi by unikalny dla kadej obsugi zdarzenia danego typu i moe by
dowolnym acuchem znakw. Jesli nazwa <i>procedury</i> nie zostanie podana,
komenda zwraca nazwe procedury ju przyporzdkowanej do danego zdarzenia
i identyfikatora, o ile takowa istnieje, w przeciwnym wypadku zwraca pusty acuch.
Proba przyporzadkowania nowej procedury, do <i>typu</i> i <i>identyfikatora</i>
majcych ju przyporzdkowanych jak procedure zakoczy si zwrceniem bdu
(dlatego w razie, gdy nie ma pewnoci, warto to sprawdzi wczeniej komend
on bez ostatniego argumentu). Jeli zostanie uyty przecznik -delete,
to argument <i>procedury</i> nie jest potrzebny, a obsuga zdarzenia dla podanego
<i>typu</i>, oraz <i>identyfikatora</i> zostanie usunita (o ile istnieje, bo jeli nie,
to zwrcony zostanie bd).<br>

<center><b>Obsugiwane <i>typy</i> zdarze</b><br></center><br>
<font color=#555555>
<b>typ_zdarzenia</b> <i>opcjonalne_argumenty[<b>spodziewany_typ_danych</b>]</i><br>
Opis zdarzenia.<br>
</font>

EVENTS

DESCRIPTION:
Podane argumenty bd przekazywane do procedury, ktr
przyporzdkujemy do obsugi tego zdarzenia. W nawiasach kwadratowych podane s typy danych, jakich
naley si spodziewa w danym argumencie.<br>

EXAMPLE:
on chstatus 0 chstat

proc chstat {uin status descr ip port} {
    set st "Uytkownik o UINie $uin zmieni status na: $status"
    if {"$descr" != ""} {
        append st " ($descr)"
    }
    append st .
    puts $st
}
ENDOFEXAMPLE

Efekt bdzie widoczny jedynie przy uruchomieniu Kadu spod konsoli, gdy komenda <b>puts</b> korzysta z wyjcia standartowego.
*/
int Tcl_CmdEvent(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2 || argc > 4)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "on ?-list? ?type ?-delete? id ?proc??");
        return TCL_ERROR;
    }
    bool act = FALSE;
    QString cmd, type = argv[1], id, key;
    int sessID = *((int*) clientData);
    TclSession *tsess = tclSess[sessID];

    if (!type.compare("-list"))
    {
        if (argc != 2)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "on ?-list? ?type ?-delete? id ?proc??");
            return TCL_ERROR;
        }
        QString filter;
        QMap<QString,QString>::Iterator it;
        for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
        {
            Tcl_AppendElement(interp, QStringList::split(":", it.key()).join(" ").local8Bit());
        }
        return TCL_OK;
    }
    else if (!QString::compare(argv[2], "-delete"))
    {
        if (argc < 3)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "on ?-list? ?type ?-delete? id ?proc??");
            return TCL_ERROR;
        }
        act = TRUE;
        cmd = argv[4];
        id = argv[3];
    }
    else
    {
        cmd = argv[3];
        id = argv[2];
    }
    QStringList events = QStringList::split(" ", "connect connfail connbroke disconnect chstatus msg chat0 chat1 chat2 \
                                                  ack dcc pubdir listreply newchat closechat sendmsg1 sendmsg2 msgsent \
                                                  startsess sessend unloadmodule chatkey searchdone pubdirnext uslmod \
                                                  usldns usladd usl");
    if (events.findIndex(type) == -1)
    {
        QCString utf = QObject::tr("Wrong event type: %1").arg(type).utf8();
        Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
        return TCL_ERROR;
    }
    key = type + ":" + id;
    if (argc == 4)
    {
        if (act)
        {
            if (tsess->tclEventList.contains(key))
            {
                tsess->tclEventList.remove(key);
            }
            else
            {
                QCString utf = QObject::tr("Event handler for \'%1\' with ident \'%2\' doesn't exist.").arg(type).arg(id).utf8();
                Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
        }
        else
        {
            if (tsess->tclEventList.contains(key))
            {
                QCString utf = QObject::tr("Event handler for \'%1\' with ident \'%2\' already exists.").arg(type).arg(id).utf8();
                Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            tsess->tclEventList[key] = argv[3];
        }
    }
    else
    {
        if (act)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "on type ?-delete? id ?proc?");
            return TCL_ERROR;
        }
        if (tsess->tclEventList.contains(key))
        {
            Tcl_SetResult(interp, (char*)tsess->tclEventList[key].latin1(), TCL_VOLATILE);
        }
    }
    return TCL_OK;
}

/* Tcl Command msg
SYNTAX: msg UINy wiadomo
DESCRIPTION:
Wysya wiadomo do kontaktu(-w) o podanym(-ych) <i>UIN'ie(-ach)</i> o treci
wiadomo. Wywoanie tej komendy nie powoduje ani otwarcia okna rozmowy z dan osob,
ani te nie dodaje tej wiadomoci do otwartego okna. Zwraca liczb, ktora stanowi identyfikator wiadomoci, ktry
mona potem wykorzysta przy zdarzeniu <b>ack</b> (patrz komenda <a href="on.html"><b>on</b></a>).
*/
int Tcl_CmdMsg(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc != 3)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "msg UINs message");
        return TCL_ERROR;
    }
    if (getCurrentStatus() == GG_STATUS_NOT_AVAIL)
    {
        QCString utf = QObject::tr("Application encountered network error.").utf8();
        Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
        return TCL_ERROR;
    }
    QString rcpt = QString::fromUtf8(argv[1]), msg = QString::fromUtf8(argv[2]);
    bool conv;
    uint uin;
    UinsList uins;
    if (rcpt.find(" ") > -1)
    {
        QCString utf;
        Tcl_Obj* result;
        QStringList list = QStringList::split(" ", rcpt);

        for ( uint i = 0; i < list.count(); i++ )
        {
            uin = list[i].toUInt(&conv);
            if (conv)
            {
                if (!uins.contains(uin))
                {
                    uins.append(uin);
                }
            }
            else
            {
                utf = QObject::tr("User isn't registered: %1\nwhile executing:\nmsg %2 %3").arg(list[i]).arg(rcpt).arg(msg).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
        }
    }
    else
    {
        uint uin = rcpt.toUInt(&conv);
        if (conv)
        {
            uins.append(uin);
        }
        else
        {
            if (userlist.containsAltNick(rcpt))
            {
                uins.append(userlist.byAltNick(rcpt).uin);
            }
            else
            {
                QCString utf = QObject::tr("User isn't registered: %1\nwhile executing:\nmsg %1 %2").arg(rcpt).arg(msg).utf8();
                Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
        }
    }
    Tcl_SetResult(interp, (char*)QString::number(gadu->sendMessage(uins, unicode2cp(msg))).latin1(), TCL_VOLATILE);
    return TCL_OK;
}

/* Tcl Command status
SYNTAX: status ?nowyStatus ?opis??
SYNTAX: status ?-showdesc?

DESCRIPTION:
Bez argumentw zwraca aktualny status lokalnego uytkownika. Mona poda opcje -showdesc, aby pobra
ewentualny opis do statusu. Jeli w takim wypadku uytkownika ma ustawiony opis, to zwrcony on bdzie jako
drugi wyraz listy (pierwszy to status). Jeli podany jest <i>nowy status</i>,
to zostanie on ustawiony. Jeli podany <i>status</i> wymaga opisu, to ostatnio ustawiany zostanie tutaj
uyty, chyba e uytkownik poda tutaj nowy opis.
*/
int Tcl_CmdStatus(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 4)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "status ?-showdesc? ?newStatus ?description??");
        return TCL_ERROR;
    }
    if ((argc == 2 || argc == 3) && QString::compare(argv[1], "-showdesc"))
    {
        QString stat = argv[1];
        int ggstat = TclMisc::TclStatusToGGStatus(stat);
        if (ggstat == -1)
        {
            QCString utf = QObject::tr("valid statuses: %1").arg("notavailable available busy available+description busy+description notavailable+description invisible invisible+description").utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
        }
        if (argc == 3)
        {
            own_description = QString::fromUtf8(argv[2]);
        }
        if (ggstat == GG_STATUS_NOT_AVAIL)
        {
            gadu->logout();
            AutoConnectionTimer::off();
            kadu->setCurrentStatus(GG_STATUS_NOT_AVAIL);
        }
        else if (ggstat == GG_STATUS_NOT_AVAIL_DESCR)
        {
            kadu->setStatus(GG_STATUS_NOT_AVAIL_DESCR);
            statusppm->setItemEnabled(7, false);
            gadu->logout();
            AutoConnectionTimer::off();
        }
        else
        {
            kadu->setStatus(ggstat);
        }
    }
    else
    {
        int status = getCurrentStatus();
        QString _status;
        bool sw1 = FALSE, sw2 = FALSE;
        if (!QString::compare(argv[1], "-showdesc"))
            sw1 = TRUE;

        _status = TclMisc::GGStatusToTclStatus(status);
        if (_status.isEmpty())
        {
            QCString utf = QObject::tr("Unknown status type.").utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
        }
        
        if (_status.find("+") > -1)
            sw2 = TRUE;

        Tcl_SetResult(interp, (char*)_status.latin1(), TCL_VOLATILE);
        if (sw1 && sw2)
        {
            QCString utf = own_description.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_AppendElement(interp, Tcl_GetString(result));
        }
    }
    return TCL_OK;
}

/* Tcl Command console
SYNTAX: console
DESCRIPTION:
Otwiera okno konsoli TCL, gdzie mona wpisywa komendy na bierzco.
*/
int Tcl_CmdConsole(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 1)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "console");
        return TCL_ERROR;
    }
    tclSess[0]->openConsole();
    return TCL_OK;
}

/* Tcl Command qlistbox
SYNTAX: qlistbox tytu ?wiadomo? lista

DESCRIPTION:
Wywietla okno o podanym <i>tytule</i>, z list sw podanych w <i>liscie</i>. Opcjonalnie
moe byc podana tre <i>wiadomoci</i>, ktra zostanie wywietlona tu nad list. W oknie
oprcz listy znajduje si pole edycji, oraz 4 przyciski: OK i CANCEL o dziaaniu wiadomym, oraz ADD i DELETE,
ktre odpowiednio: dodaj warto pola edycji i uswaj zaznaczony element z listy. Komenda
zwraca w zalenoci od potwierdzenia, bd anulowania operacji list: po dokonanych operacjach na liscie, bd
jej stan pierwotny, z jakim zostaa wywoana.
*/
int Tcl_CmdListBox(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 4 || argc < 3)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "listbox caption ?message? list");
        return TCL_ERROR;
    }
    int _argc, code;
    const char **_argv;
    QString caption = QString::fromUtf8(argv[1]), message, tmp;
    QStringList list;
    QCString utf;
    Tcl_Obj* result;
    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    
    if (argc == 3)
    {
        code = Tcl_SplitList(interp, argv[2], &_argc, &_argv);
    }
    else
    {
        code = Tcl_SplitList(interp, argv[3], &_argc, &_argv);
        message = QString::fromUtf8(argv[2]);
    }
    if (code != TCL_OK)
    {
        utf = QObject::tr("Invalid list argument for command listbox.").utf8();
        result = Tcl_NewStringObj((const char* const)utf, utf.length());
        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
        return code;
    }
    TclListBox *dialog;
    if (!message.isNull())
    {
        dialog = new TclListBox(TRUE);
        dialog->setMsg(message);
    }
    else
    {
        dialog = new TclListBox(FALSE);
    }
    dialog->setCaption(caption);
    for (int i = 0; i < _argc; i++ )
    {
        tmp = QString::fromUtf8(_argv[i]);
        dialog->list()->insertItem(tmp);
        list.append(tmp);
    }

    dialog->show();
    if (dialog->exec() == QDialog::Accepted)
    {
        int cnt = dialog->list()->count();
        for ( int i = 0; i < cnt; i++ )
        {
            utf = dialog->list()->text(i).utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));
        }
    }
    else
    {
        for ( QStringList::iterator it = list.begin(); it != list.end(); it++ )
        {
            utf = QString(*it).utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));
        }
    }
    Tcl_DStringResult(interp, &ds);
    Tcl_DStringFree(&ds);
    delete dialog;
    return TCL_OK;
}

/* Tcl Command menu
GLOBALSYNTAX: qmenu komenda nazwa ?argumenty?
DESCRIPTION:
Komenda do tworzenia i zarzdzania menu.

SUBSYNTAX: qmenu new nazwa
DESCRIPTION:
Tworzy nowe menu. Nazwa jest jednoczesnie identyfikatorem uzywanym w pozostaych podkomendach menu,
tytuem menu wywietlanego w oknie Kadu, jak i nazw menu popup, ktry zostanie przyporzdkowany do tego menu.
Popup menu musi by oczywicie uprzednio stworzone za pomoc komendy <b>popup</b>. Jeli menu o podanej
<i>nazwie</i> ju istnieje, zwraca bd. Uycie znaku <b>&amp;</b> gdzie w <i>nazwie</i> (ale nie na kocu)
spowoduje automatyczne przydzielenie skrtu klawiszowego Alt+&lt;litera znajdujca si tu za tym znakiem&gt;
do otwierania tego menu. W pniejszym uywaniu <i>nazwy</i> manu, naley uwzgldni uyty znak <b>&amp;</b>.

SUBSYNTAX: qmenu delete nazwa
DESCRIPTION:
Usuwa menu o podanej nazwie. Jeli menu o podanej <i>nazwie</i> nie istnieje, zwraca bd.

SUBSYNTAX: qmenu exists nazwa
DESCRIPTION:
Jeli menu o <i>nazwie</i> istnieje, zwraca 1, inaczej zwraca 0.

SUBSYNTAX: qmenu disable nazwa
DESCRIPTION:
Wycza menu. W takim stanie uytkownik nie moe na nim klikn, a kolor jego tytuu zmienia si na janiejszy.
Jeli menu o podanej <i>nazwie</i> nie istnieje, zwraca bd.

SUBSYNTAX: qmenu enable nazwa
DESCRIPTION:
Wcza menu. Jest to domylny tryb pracy menu zaraz po stworzeniu.
Jeli menu o podanej <i>nazwie</i> nie istnieje, zwraca bd.

SUBSYNTAX: qmenu isenabled nazwa
DESCRIPTION:
Zwraca 1, jeli menu o podanej <i>nazwie</i> jest wczone, inaczej zwraca 0.
*/
int Tcl_CmdMenu(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "qmenu command name ?arguments?");
        return TCL_ERROR;
    }
    QStringList cmds = QStringList::split(" ", "new delete disable enable exists isenabled");
    QString command = argv[1], err, icon, name = QString::fromUtf8(argv[2]);
    QCString utf;
    Tcl_Obj* result;
    int sessID = *((int*) clientData);
    TclSession *tsess = tclSess[sessID];

    switch (cmds.findIndex(command))
    {
        case 0:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "qmenu new name");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                tsess->tclMenuBar[name] = kadu->menuBar()->insertItem(name, tsess->tclPopups[name].menu);
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 1:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "qmenu delete name");
                return TCL_ERROR;
            }
            if (tsess->tclMenuBar.contains(name))
            {
                kadu->menuBar()->removeItem(tsess->tclMenuBar[name]);
            }
            else
            {
                utf = QObject::tr("Menu %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 2:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "qmenu disable name");
                return TCL_ERROR;
            }
            if (tsess->tclMenuBar.contains(name))
            {
                if (kadu->menuBar()->isItemEnabled(tsess->tclMenuBar[name]))
                {
                    kadu->menuBar()->setItemEnabled(tsess->tclMenuBar[name], FALSE);
                }
                else
                {
                    utf = QObject::tr("Menu '%1' is already disabled.").arg(name).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
            }
            else
            {
                utf = QObject::tr("Menu %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 3:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "qmenu enable name");
                return TCL_ERROR;
            }
            if (tsess->tclMenuBar.contains(name))
            {
                if (!kadu->menuBar()->isItemEnabled(tsess->tclMenuBar[name]))
                {
                    kadu->menuBar()->setItemEnabled(tsess->tclMenuBar[name], TRUE);
                }
                else
                {
                    utf = QObject::tr("Menu '%1' is already enabled.").arg(name).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
            }
            else
            {
                utf = QObject::tr("Menu %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 4:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "qmenu exists name");
                return TCL_ERROR;
            }
            if (tsess->tclMenuBar.contains(name))
            {
                Tcl_SetResult(interp, "1", TCL_STATIC);
            }
            else
            {
                Tcl_SetResult(interp, "0", TCL_STATIC);
            }
            break;
        }
        case 5:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "qmenu isenabled name");
                return TCL_ERROR;
            }
            if (tsess->tclMenuBar.contains(name))
            {
                if (kadu->menuBar()->isItemEnabled(tsess->tclMenuBar[name]))
                {
                    Tcl_SetResult(interp, "1", TCL_STATIC);
                }
                else
                {
                    Tcl_SetResult(interp, "0", TCL_STATIC);
                }
            }
            else
            {
                utf = QObject::tr("Menu %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        default:
            utf = QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")).utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
    }
    return TCL_OK;
}

/* Tcl Command config
GLOBALSYNTAX: config komenda argumenty
DESCRIPTION:
Tworzy nowe wpisy w oknie konfiguracyjnym. Kazdy taki wpis zapisuje jednoczesnie swoja wartosc w pliku
konfiguracyjnym, tak wiec wartosci tworzonych widgetow nalezy odczytywac z pliku konfiguracyjnego za pomoca
komendy configfile (gdzie nazw grupy bdzie nazwa zakadki z okna konfiguracyjnego,
a nazw zmiennej bdzie stanowi ta podana w config add).

SUBSYNTAX: config newtab nazwa
DESCRIPTION:
Dodaje now zakadk o podanej <i>nazwie</i> do okna konfoguracji. Niestety nie sprawdza,
czy zakadka o tej <i>nazwie</i> ju istnieje, wic podczas pierwszego tworzenia tej zakadki,
trzeba doda te chocia jedn kontrolk (patrz niej), eby mona byo potem uy komendy
config exists odnonie tej kontrolki, co zarazem bdzie wiadczy o istnieniu danej zakadki.

SUBSYNTAX: config add -type typ -tab zakadka -caption tytu ?-parent rodzic? ?-var nazwaZmiennej? ?-default warto? ?-hint podpowied? ?-cols liczba?
DESCRIPTION:
Dodaje now kontrolk danego <i>typu</i> do zakadki o podanej <i>nazwie</i>. Kontrolka ta otrzyma dany
tytu. Mona doda jeden z piciu typw kontrolek: <b>checkbox</b>, <b>label</b>, <b>lineedit</b>,
<b>vbox</b>, <b>hbox</b>, lub <b>grid</b>. Argument <i>nazwy zmiennej</i> jest wymagany tylko do <i>typw</i>:
<b>checkbox</b>, <b>lineedit</b>, oraz <b>color</b>, ktre rwnierz mog pobra opcjonalne argumenty - <i>domyln warto</i>,
oraz podpowied, ktra ukae si w tzw. dymku. Kontrolki te zapisuj w pliku konfiguracyjnym,
w grupie o nazwie identycznej, do <i>nazwy zakadki</i>, w zmiennej o podanej <i>nazwie</i>. <i>Domylna warto</i>,
jest ustawiana dla kontrolki, zaraz po jej utworzeniu, lub gdy jej wartoci nie mona byo odczyta z pliku konfiguracyjnego.
Typ <b>label</b> suy jedynie informacji i nie wymaga ani <i>nazwy zmiennej</i>, ani <i>domylnej wartoci</i>,
ani te <i>podpowiedzi</i>. Podobnie rzecz si ma z typami <b>vbox</b>, oraz <b>hbox</b>, ktre tworz ramki,
dzielc kontrolki na zatytuowane (przez ramki) podgrupy. <b>vbox</b> tworzy ramk pionow, za <b>hbox</b> poziom.
Typ <b>grid</b> tworzy niewidoczn siatk, w ktrej mona umieszcza swoje kontrolki. W tym wypadku niezbdne jest podanie
liczby <i>kolumn</i>. Kady dodawany element bdzie si znajdowa w pierwszym wolnym wierszu, w ktrym s jeszcze wolne kolumny.
Opcja <b>-parent</b> suy do "wsadzania" nowych kontrolek do danej ramki. <i>rodzic</i> to nic innego, jak nazwa (tytu)
poprzednio utworzonej ramki. Jeli podczas dodawania nowej kontrolki uyjemy <i>rodzica</i> (ktrym <u>musi</u> by tytu ramki)
to owa kontrolka zostanie zagniedona automatycznie w tej ramce.

SUBSYNTAX: config delete nazwaZakadki ?tytu?
DESCRIPTION:
Usuwa kontrolk z danej <i>zakadki</i> o podanym <i>tytule</i>. Jeli tytu nie jest podany,
usuwana jest zakadka o podanej <i>nazwie</i>. Aby usun zakadk, wczeniej trzeba usun wszystkie
kontrolki w niej zawarte.

SUBSYNTAX: config exists nazwaZakadki tytu
DESCRIPTION:
Sprawdza czy kontrolka o <i>tytule</i> w zakadce o <i>nazwie</i> istnieje. Jeli tak, zwraca 1, jeli nie, zwraca 0.
*/
int Tcl_CmdConfig(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "config command arguments");
        return TCL_ERROR;
    }
    QStringList cmds = QStringList::split(" ", "newtab add exists delete"),
                types = QStringList::split(" ", "checkbox label lineedit vbox hbox grid"),
                opts = QStringList::split(" ", "-type -tab -caption -parent -var -default -hint -cols");
    QString command = argv[1], grp, cap, var, err, type, strDef, parent, hint, tmp, syntax, cols;
    int res, boolDef, id, _cols;
    QCString utf;
    Tcl_Obj* result;

    switch (cmds.findIndex(command))
    {
        case 0:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "config newtab tabName");
                return TCL_ERROR;
            }
            grp = QString::fromUtf8(argv[2]);
            if (ConfigDialog::findTab(grp) > -1)
            {
                utf = QObject::tr("Tab %1 already exists").arg(grp).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            ConfigDialog::addTab(grp);
            break;
        }
        case 1:
        {
            syntax = "config add -type type -tab tabName -caption caption ?-parent parent? ?-var varName? ?-default value? ?-hint text? ?-cols number?";
            res = argc % 2;
            if (argc < 8 || res)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                return TCL_ERROR;
            }
            for ( int i = 2; i < argc; i++ )
            {
                id = opts.findIndex(argv[i]);
                i++;
                tmp = QString::fromUtf8(argv[i]);
                switch (id)
                {
                    case 0:
                        type = tmp;
                        break;
                    case 1:
                        grp = tmp;
                        break;
                    case 2:
                        cap = tmp;
                        break;
                    case 3:
                        parent = tmp;
                        break;
                    case 4:
                        var = tmp;
                        break;
                    case 5:
                        strDef = tmp;
                        break;
                    case 6:
                        hint = tmp;
                        break;
                    case 7:
                        cols = tmp;
                        break;
                    default:
                        Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                        return TCL_ERROR;
                }
            }
            if (type.isNull() || grp.isNull() || cap.isNull())
            {
                Tcl_WrongNumArgs(interp, 0, NULL, syntax.local8Bit());
                return TCL_ERROR;
            }
            if (ConfigDialog::findTab(grp) == -1)
            {
                utf = QObject::tr("Tab %1 doesn't exists").arg(grp).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }

            switch (types.findIndex(type))
            {
                case 0:
                {
                    if (!strDef.isNull())
                    {
                        res = Tcl_GetBoolean(interp, strDef.latin1(), &boolDef);
                        if (res == TCL_ERROR)
                        {
                            utf = QObject::tr("Default value for checkbox should be boolean type.").utf8();
                            result = Tcl_NewStringObj((const char* const)utf, utf.length());
                            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                            return TCL_ERROR;
                        }
                    }
                    else
                    {
                        boolDef = FALSE;
                    }
                    if (var.isNull())
                    {
                        utf = QObject::tr("Checkbox type needs a variable name to be given.").utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                    if (parent.isNull())
                    {
                        parent = grp;
                    }
                    if (ConfigDialog::existControl(grp, cap) == -1)
                    {
                        ConfigDialog::addCheckBox(grp, parent, cap, var, boolDef, hint);
                    }
                    else
                    {
                        utf = QObject::tr("checkbox %1 in tab %2 already exists.").arg(cap).arg(grp).utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                    break;
                }
                case 1:
                {
                    if (parent.isNull())
                    {
                        parent = grp;
                    }
                    if (ConfigDialog::existControl(grp, cap) == -1)
                    {
                        ConfigDialog::addLabel(grp, parent, cap);
                    }
                    else
                    {
                        utf = QObject::tr("label %1 in tab %2 already exists.").arg(cap).arg(grp).utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                    break;
                }
                case 2:
                {
                    if (var.isNull())
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, "Lineedit type needs a variable name to be given.");
                        return TCL_ERROR;
                    }
                    if (parent.isNull())
                    {
                        parent = grp;
                    }
                    if (ConfigDialog::existControl(grp, cap) == -1)
                    {
                        ConfigDialog::addLineEdit(grp, parent, cap, var, strDef, hint);
                    }
                    else
                    {
                        utf = QObject::tr("Lineedit %1 in tab %2 already exists.").arg(cap).arg(grp).utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                    break;
                }
                case 3:
                {
                    if (parent.isNull())
                    {
                        parent = grp;
                    }
                    if (ConfigDialog::existControl(grp, cap) == -1)
                    {
                        ConfigDialog::addVGroupBox(grp, parent, cap);
                    }
                    else
                    {
                        utf = QObject::tr("vbox %1 in tab %2 already exists.").arg(cap).arg(grp).utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                    break;
                }
                case 4:
                {
                    if (parent.isNull())
                    {
                        parent = grp;
                    }
                    if (ConfigDialog::existControl(grp, cap) == -1)
                    {
                        ConfigDialog::addHGroupBox(grp, parent, cap);
                    }
                    else
                    {
                        utf = QObject::tr("hbox %1 in tab %2 already exists.").arg(cap).arg(grp).utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                    break;
                }
                case 5:
                {
                    if (!cols.isNull())
                    {
                        res = Tcl_GetBoolean(interp, cols.latin1(), &_cols);
                        if (res == TCL_ERROR)
                        {
                            Tcl_WrongNumArgs(interp, 0, NULL, "Columns number should be an integer value.");
                            return TCL_ERROR;
                        }
                    }
                    else
                    {
                        _cols = 3;
                    }
                    if (parent.isNull())
                    {
                        parent = grp;
                    }
                    if (ConfigDialog::existControl(grp, cap) == -1)
                    {
                        ConfigDialog::addGrid(grp, parent, cap, _cols);
                    }
                    else
                    {
                        utf = QObject::tr("grid %1 in tab %2 already exists.").arg(cap).arg(grp).utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                    break;
                }
                default:
                    utf = QObject::tr("Type should be one of following:\n%1").arg(types.join(", ")).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
            }
            break;
        }
        case 2:
        {
            if (argc > 4 || argc < 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "config exists tabName ?caption?");
                return TCL_ERROR;
            }
            grp = QString::fromUtf8(argv[2]);
            if (argc == 4)
            {
                cap = QString::fromUtf8(argv[3]);
                if (ConfigDialog::existControl(grp, cap) > -1)
                {
                    Tcl_SetResult(interp, "1", TCL_STATIC);
                }
                else
                {
                    Tcl_SetResult(interp, "0", TCL_STATIC);
                }
            }
            else
            {
                if (ConfigDialog::findTab(grp) > -1)
                {
                    Tcl_SetResult(interp, "1", TCL_STATIC);
                }
                else
                {
                    Tcl_SetResult(interp, "0", TCL_STATIC);
                }
            }
            break;
        }
        case 3:
        {
            if (argc > 4 || argc < 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "config delete tabName ?caption?");
                return TCL_ERROR;
            }
            grp = QString::fromUtf8(argv[2]);
            if (argc == 4)
            {
                cap = QString::fromUtf8(argv[3]);
                if (!ConfigDialog::existControl(grp, cap) > -1)
                {
                    err = QString("Widget %1 (in %2) doesn't exist.").arg(cap).arg(grp);
                    Tcl_WrongNumArgs(interp, 0, NULL, err.local8Bit());
                    return TCL_ERROR;
                }
                ConfigDialog::removeControl(grp, cap);
            }
            else
            {
                if (ConfigDialog::findTab(grp) == -1)
                {
                    utf = QObject::tr("Tab %1 doesn't exists").arg(grp).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
                ConfigDialog::removeTab(grp);
            }
            break;
        }
        default:
            utf = QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")).utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
    }
    return TCL_OK;
}
/*
Nie mozna zidentyfikowac po sygnale, ktory button zostal wcisniety, wiec nie mozna przyporzadkowac
odpowiedniego do niego kodu.

int Tcl_CmdToolBar(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "config command arguments");
        return TCL_ERROR;
    }
    QStringList cmds = QStringList::split(" ", "add delete exists");
    switch (cmds.findIndex(argv[1]))
    {
        case 0:
        {
            if (argc < 2)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "config add ");
                return TCL_ERROR;
            }
            Kadu::addToolButton();
            break;
        }
        case 1:
        {
            break;
        }
        case 2:
        {
            break;
        }
        default:
        {
        }
    }
    return TCL_OK;
}
*/

/* Tcl Command usermenu
GLOBALSYNTAX: usermenu komenda ?argumenty?
DESCRIPTION:
Komenda do zarzdzania menu kontekstowym listy kontaktw.

SUBSYNTAX: usermenu add nazwa ?komenda? ?ikona?
DESCRIPTION:
Dodaje nowy element do menu o podanej <i>nazwie</i>, ktremu przypisana zostanie
podana komenda. Opcjonalnie mona poda <i>ikon</i>. Element moe by rwnierz nazw
popup menu, utowrzonego wczeniej komend <b>popup</b>. Wtedy argument <i>komendy</i>
zostaje pominity.

SUBSYNTAX: usermenu delete nazwa
DESCRIPTION:
Usuwa element menu o podanej nazwie. Jeli menu o podanej <i>nazwie</i> nie istnieje, zwraca bd.<br>

SUBSYNTAX: usermenu exists nazwa
DESCRIPTION:
Jeli element menu o <i>nazwie</i> istnieje, zwraca 1, inaczej zwraca 0.

SUBSYNTAX: usermenu disable nazwa
DESCRIPTION:
Wycza element menu. W takim stanie uytkownik nie moe na nim klikn, a kolor jego tytuu zmienia si na janiejszy.
Jeli element o podanej <i>nazwie</i> nie istnieje, zwraca bd.<br>

SUBSYNTAX: usermenu enable nazwa
DESCRIPTION:
Wcza element menu. Jest to domylny tryb pracy menu zaraz po stworzeniu.
Jeli element o podanej <i>nazwie</i> nie istnieje, zwraca bd.<br>

SUBSYNTAX: usermenu isenabled nazwa
DESCRIPTION:
Zwraca 1, jeli element menu o podanej <i>nazwie</i> jest wczone, inaczej zwraca 0.

SUBSYNTAX: usermenu listitems
DESCRIPTION:
Zwraca nazwy wszystkich elementw z menu w postaci listy TCL.

SUBSYNTAX: usermenu selected ?-altname?
DESCRIPTION:
Zwraca UIN, lub list UIN'w wybranych przez uytkownika na licie kontaktw. Jeli podana zostanie
opcja -altname, zwracane elementy to nazwy alternatywne (wywietlane), zamiast UIN'w.
*/
int Tcl_CmdUserMenu(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "usermenu command ?arguments?");
        return TCL_ERROR;
    }
    QStringList cmds = QStringList::split(" ", "add delete exists disable enable list selected");
    QString name = QString::fromUtf8(argv[2]), err, icon;
    int id = UserBox::userboxmenu->getItem(name);
    QCString utf;
    Tcl_Obj* result;
    int sessID = *((int*) clientData);
    TclSession *tsess = tclSess[sessID];

    switch (cmds.findIndex(argv[1]))
    {
        case 0:
        {
            if (tsess->tclPopups.contains(name))
            {
                if (argc < 3 || argc > 4)
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, "usermenu add name ?command? ?icon?");
                    return TCL_ERROR;
                }
                if (argc == 4)
                {
                    icon = QString::fromUtf8(argv[3]);
                }
                id = UserBox::userboxmenu->insertItem(icons_manager.loadIcon(icon), name, tsess->tclPopups[name].menu);
            }
            else
            {
                if (argc < 4 || argc > 5)
                {
                    Tcl_WrongNumArgs(interp, 0, NULL, "usermenu add name ?command? ?icon?");
                    return TCL_ERROR;
                }
                if (argc == 5)
                {
                    icon = QString::fromUtf8(argv[4]);
                }
                id = UserBox::userboxmenu->insertItem(icons_manager.loadIcon(icon), QString::fromUtf8(argv[2]), tsess, SLOT(handleUserMenu(int)));
                tsess->tclMenuCmds[id] = QString::fromUtf8(argv[3]);
            }
            break;
        }
        case 1:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "usermenu delete name");
                return TCL_ERROR;
            }
            if (id != -1)
            {
                UserBox::userboxmenu->removeItem(id);
                tsess->tclMenuCmds[id].remove(id);
            }
            else
            {
                utf = QObject::tr("User menu item '%1' doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 2:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "usermenu exists name");
                return TCL_ERROR;
            }
            if (id != -1)
            {
                Tcl_SetResult(interp, "1", TCL_STATIC);
            }
            else
            {
                Tcl_SetResult(interp, "0", TCL_STATIC);
            }
            return TCL_OK;
            break;
        }
        case 3:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "usermenu disable name");
                return TCL_ERROR;
            }
            if (id != -1)
            {
                if (UserBox::userboxmenu->isItemEnabled(id))
                {
                    UserBox::userboxmenu->setItemEnabled(id, FALSE);
                }
                else
                {
                    utf = QObject::tr("User menu item '%1' is already disabled.").arg(name).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
            }
            else
            {
                utf = QObject::tr("User menu item '%1' doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 4:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "usermenu enable name");
                return TCL_ERROR;
            }
            if (id != -1)
            {
                if (!UserBox::userboxmenu->isItemEnabled(id))
                {
                    UserBox::userboxmenu->setItemEnabled(id, TRUE);
                }
                else
                {
                    utf = QObject::tr("User menu item '%1' is already enabled.").arg(name).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
            }
            else
            {
                utf = QObject::tr("User menu item '%1' doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 5:
        {
            if (argc != 2)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "usermenu list");
                return TCL_ERROR;
            }
            for ( uint i = 0; i <= UserBox::userboxmenu->count(); i++ )
            {
                utf = UserBox::userboxmenu->text(UserBox::userboxmenu->idAt(i)).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                if (utf.length())
                    Tcl_AppendElement(interp, Tcl_GetString(result));
            }
            return TCL_OK;
            break;
        }
        case 6:
        {
            if (argc > 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "usermenu selected ?-altname?");
                return TCL_ERROR;
            }
            bool alt = FALSE;
            if (argc == 3)
            {
                if (!QString::compare("-altname", argv[2]))
                    alt = TRUE;
            }
            if (kadu->userbox()->getSelectedUins().count())
            {
                if (alt)
                {
                    QStringList uins = kadu->userbox()->getSelectedAltNicks();
                    for ( uint i = 0; i < uins.count(); i++ )
                    {
                        utf = uins[i].utf8();
                        result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_AppendElement(interp, Tcl_GetString(result));
                    }
                }
                else
                {
                    UinsList uins = kadu->userbox()->getSelectedUins();
                    for ( uint i = 0; i < uins.count(); i++ )
                        Tcl_AppendElement(interp, (char*)QString::number(uins[i]).latin1());
                }
            }
            return TCL_OK;
        }
        default:
            utf = QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")).utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
    }
    return TCL_OK;
}

/* Tcl Command popup
GLOBALSYNTAX: popup komenda ?nazwa? ?argumenty?
DESCRIPTION:
popup suy do tworzenia gotowego do uycia menu w pamici. Takowe menu moe zosta
uyte w dowolnym miejscu (w ktrym wchodzi w gre menu), jak na belce gwnego okna Kadu,
lub w menu kontekstowym listy kontaktw. Jeli chce si go uy w wielu miejscach jednoczenie,
nic nie stoi na przeszkodzie, ani te nie trzeba budowa go od nowa. Istnieje ono cay czas w pamici,
a do usunicia komend popup <b>delete</b>.

SUBSYNTAX: popup new nazwa
DESCRIPTION:
Tworzy nowe menu o podanej <i>nazwie</i>.

SUBSYNTAX: popup delete nazwa
Niszczy menu i zwalnia pami. Wczeniej naley usun wszelkie wystpienia tego menu
w aplikacji.

SUBSYNTAX: popup exists nazwa
DESCRIPTION:
Zwraca 1 jeli menu o podanej <i>nazwie</i> istnieje, inaczej zwraca 0.

SUBSYNTAX: popup list
DESCRIPTION:
Zwraca list nazw menu istniejcych w pamici.

SUBSYNTAX: popup listitems nazwa
DESCRIPTION:
Zwraca list elementw zawartych w menu o podanej <i>nazwie</i>.

SUBSYNTAX: popup additem nazwa tytu komenda ?ikona ?pozycja ?skrt???
DESCRIPTION:
Dodaje nowy element o danym <i>tytule</i> do menu o <i>nazwie</i>. Po wybraniu tego elementu
przez uytkownika, wykonana zostanie komenda. Opcjonalnie mona poda nazw ikony dla elementu,
pozycj na ktrej ma znajdowa si nowoutworzony element, oraz skrt klawiszowy.

SUBSYNTAX: popup addseparator nazwa
DESCRIPTION:
Dodaje belk oddzielajc do menu o podanej <i>nazwie</i>.

SUBSYNTAX: popup delitem nazwa tytu
DESCRIPTION:
Usuwa element o podanym <i>tytule</i> z menu o podanej <i>nazwie</i>.

SUBSYNTAX: popup delindex nazwa indeks
DESCRIPTION:
Usuwa element o numerze podanym jako indeks, liczc od 0, z menu o podanej <i>nazwie</i>.

SUBSYNTAX: popup disableitem nazwa tytu
DESCRIPTION:
Wycza element o <i>tytule</i> w menu o <i>nazwie</i>.

SUBSYNTAX: popup enableitem nazwa tytu
DESCRIPTION:
Wcza element o <i>tytule</i> w menu o <i>nazwie</i>.

SUBSYNTAX: popup isitemenabled nazwa tytu
DESCRIPTION:
Zwraca 1, jeli element o podanym <i>tytule</i> z menu o podanej <i>nazwie</i> jest wczony,
inaczej zwraca 0.

SUBSYNTAX: popup itemconf nazwa tytu ?-newname nowyTytu? ?-cmd komenda? ?-icon ikona? ?-accel skrt?
DESCRIPTION:
Rekonfiguruje ustawienia elementu tytu z menu o podanej <i>nazwie</i>, wg. uytych opcji.
Jeli adna z opcji nie jest podana, zwraca list zawierajc tytu elementu i komend do niego
przyporzdkowan w takiej wasnie kolejnoci.

SUBSYNTAX: popup addpopup nazwa nazwaPopupa ?ikona?
DESCRIPTION:
Dodaje do menu o podanej <i>nazwie</i> inne menu o <i>nazwie popupa</i>, w postaci jednego elementu.
Opcjonalnie mona poda </i>ikon<i>.
*/
int Tcl_CmdPopup(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "popup command ?name? ?arguments?");
        return TCL_ERROR;
    }
    QStringList cmds = QStringList::split(" ", "new delete exists list listitems additem addseparator delitem disableitem enableitem isitemenabled delindex addpopup itemconf");
    QString name = QString::fromUtf8(argv[2]), err, icon, name2;
    QCString utf;
    Tcl_Obj* result;
    int id, itemId = -1, sessID = *((int*) clientData);
    QPopupMenu *menu;
    TclSession *tsess = tclSess[sessID];

    if (argc > 2)
    {
        if (tsess->tclPopups.contains(name))
        {
            for ( uint i = 0; i <= tsess->tclPopups[name].menu->count(); i++ )
            {
                if ( ! tsess->tclPopups[name].menu->text(tsess->tclPopups[name].menu->idAt(i)).compare(QString::fromUtf8(argv[3])) )
                {
                    itemId = tsess->tclPopups[name].menu->idAt(i);
                    break;
                }
            }
        }
    }

    switch (cmds.findIndex(argv[1]))
    {
        case 0:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup new name");
                return TCL_ERROR;
            }
            menu = new QPopupMenu(kadu);
            if (tsess->tclPopups.contains(name))
            {
                utf = QObject::tr("Popup %1 already exists.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            tsess->tclPopups[name].menu = menu;
            QObject::connect(tsess->tclPopups[name].menu, SIGNAL(activated(int)), tsess, SLOT(handlePopup(int)));
            break;
        }
        case 1:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup delete name");
                return TCL_ERROR;
            }
            if (!tsess->tclPopups.contains(name))
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            QObject::disconnect(tsess->tclPopups[name].menu, SIGNAL(activated(int)), tsess, SLOT(handlePopup(int)));
            delete tsess->tclPopups[name].menu;
            tsess->tclPopups.remove(name);
            break;
        }
        case 2:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup exists name");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                Tcl_SetResult(interp, "1", TCL_STATIC);
            }
            else
            {
                Tcl_SetResult(interp, "0", TCL_STATIC);
            }
            break;
        }
        case 3:
        {
            if (argc != 2)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup list");
                return TCL_ERROR;
            }
            for ( uint i = 0; i < tsess->tclPopups.keys().count(); i++ )
            {
                utf = tsess->tclPopups.keys()[i];
                   result = Tcl_NewStringObj((const char* const)utf, utf.length());
                if (utf.length())
                    Tcl_AppendElement(interp, Tcl_GetString(result));
            }
            break;
        }
        case 4:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup listitems name");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                for ( uint i = 0; i <= tsess->tclPopups[name].menu->count(); i++ )
                {
                    utf = tsess->tclPopups[name].menu->text(tsess->tclPopups[name].menu->idAt(i)).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    if (utf.length())
                        Tcl_AppendElement(interp, Tcl_GetString(result));
                }
                return TCL_OK;
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 5:
        {
            if (argc < 5 || argc > 8)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup additem name itemName itemCommand ?icon ?position ?accelerator???");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                int pos = -1;
                if (argc > 5)
                {
                    icon = QString::fromUtf8(argv[5]);
                    if (argc == 7)
                    {
                        bool ok;
                        pos = QString(argv[6]).toUInt(&ok);
                        if (!ok)
                        {
                            if (QString::compare(argv[6], "end"))
                            {
                                pos = -1;
                            }
                            else
                            {
                                utf = QObject::tr("Position argument in 'popup' command should be an integer value.").utf8();
                                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                            }
                        }
                    }
                }
                id = tsess->tclPopups[name].menu->insertItem(icons_manager.loadIcon(icon), QString::fromUtf8(argv[3]), -1, pos);
                if (argc == 8)
                    tsess->tclPopups[name].menu->setAccel(QKeySequence(argv[7]), id);

                tsess->tclPopupCmds[id] = QString::fromUtf8(argv[4]);
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 6:
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup addseparator name");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                tsess->tclPopups[name].menu->insertSeparator();
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 7:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup delitem name itemName");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                if (itemId == -1)
                {
                    utf = QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
                tsess->tclPopups[name].menu->removeItem(itemId);
                tsess->tclPopupCmds[id].remove(itemId);
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 8:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup disableitem name itemName");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                if (itemId == -1)
                {
                    utf = QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
                if (tsess->tclPopups[name].menu->isItemEnabled(itemId))
                {
                    tsess->tclPopups[name].menu->setItemEnabled(itemId, FALSE);
                }
                else
                {
                    utf = QObject::tr("Popup item '%1' is already disabled.").arg(name).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 9:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup enableitem name itemName");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                if (itemId == -1)
                {
                    utf = QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
                if (!tsess->tclPopups[name].menu->isItemEnabled(itemId))
                {
                    tsess->tclPopups[name].menu->setItemEnabled(itemId, TRUE);
                }
                else
                {
                    utf = QObject::tr("Popup item '%1' is already enabled.").arg(name).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 10:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup isitemenabled name itemName");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                if (itemId == -1)
                {
                    utf = QObject::tr("Popup item '%1' doesn't exist.").arg(QString::fromUtf8(argv[3])).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
                if (tsess->tclPopups[name].menu->isItemEnabled(itemId))
                {
                    Tcl_SetResult(interp, "1", TCL_STATIC);
                }
                else
                {
                    Tcl_SetResult(interp, "0", TCL_STATIC);
                }
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 11:
        {
            if (argc != 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup delindex name index");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                int idxID = atoi(argv[3]);
                tsess->tclPopups[name].menu->removeItem( tsess->tclPopups[name].menu->idAt(idxID) );
                tsess->tclPopupCmds[id].remove(idxID);
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 12:
        {
            if (argc < 4 || argc > 5)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup addpopup name popupToAdd ?icon?");
                return TCL_ERROR;
            }
            name2 = QString::fromUtf8(argv[3]);
            if (tsess->tclPopups.contains(name))
            {
                if (tsess->tclPopups.contains(name2))
                {
                    if (argc == 5)
                    {
                        icon = QString::fromUtf8(argv[4]);
                    }
                    tsess->tclPopups[name].menu->insertItem(icons_manager.loadIcon(icon), name2, tsess->tclPopups[name2].menu);
                }
                else
                {
                    utf = QObject::tr("Popup %1 doesn't exist.").arg(name2).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                    return TCL_ERROR;
                }
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        case 13:
        {
            if (argc < 4)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "popup itemconf name itemName ?-newname itemName? ?-cmd command? ?-icon icon? ?-accel accelerator?");
                return TCL_ERROR;
            }
            if (tsess->tclPopups.contains(name))
            {
                if (argc > 4)
                {
                    bool rest = argc % 2;
                    if (rest)
                    {
                        Tcl_WrongNumArgs(interp, 0, NULL, "popup itemconf name itemName ?-newname itemName? ?-cmd command? ?-icon icon? ?-accel accelerator?");
                        return TCL_ERROR;
                    }
                    QStringList opts = QStringList::split(" ", "-newname -cmd -icon -accel");
                    QString opt, arg;
                    for ( int i = 4; i < argc; i++ )
                    {
                        opt = argv[i];
                        i++;
                        arg = QString::fromUtf8(argv[i]);
                        switch (opts.findIndex(opt))
                        {
                            case 0:
                                tsess->tclPopups[name].menu->changeItem(itemId, arg);
                                break;
                            case 1:
                                tsess->tclPopupCmds[itemId] = arg;
                                break;
                            case 2:
                                tsess->tclPopups[name].menu->changeItem(itemId, icons_manager.loadIcon(arg), tsess->tclPopups[name].menu->text(itemId));
                                break;
                            case 3:
                                tsess->tclPopups[name].menu->setAccel(QKeySequence(arg), itemId);
                                break;
                            default:
                                utf = QObject::tr("Unknown option: %1.").arg(opt).utf8();
                                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                                return TCL_ERROR;
                        }
                    }
                }
                else
                {
                    utf = tsess->tclPopups[name].menu->text(itemId).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_AppendElement(interp, Tcl_GetString(result));

                    utf = tsess->tclPopupCmds[itemId].utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_AppendElement(interp, Tcl_GetString(result));
                }
            }
            else
            {
                utf = QObject::tr("Popup %1 doesn't exist.").arg(name).utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            break;
        }
        default:
            utf = QObject::tr("Command should be one of following:\n%1").arg(cmds.join(", ")).utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
    }
    return TCL_OK;
}

/* Tcl Command trayicon
SYNTAX: trayicon ikona
DESCRIPTION:
Komenda suca ustawianiu <i>ikony</i> zadokowanego Kadu.
*/
int Tcl_CmdTrayIcon(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc != 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "trayicon icon");
        return TCL_ERROR;
    }
//    QPixmap p = icons_manager.loadIcon(argv[1]);
//    trayicon->setType(p);
    return TCL_OK;
}

/* Tcl Command parse
SYNTAX: parse UIN tekst
DESCRIPTION:
Podmienia w podanym <i>tekscie</i> specjalne znaki, na odpowiednie dane dotyczce podanego
<i>UINa</i>. Podmieniane znaki to:<br>
<table>
<tr><td><b>%s</b></td><td>- status</td></tr>
<tr><td><b>%d</b></td><td>- opis</td></tr>
<tr><td><b>%i</b></td><td>- adres ip</td></tr>
<tr><td><b>%v</b></td><td>- adres dns</td></tr>
<tr><td><b>%n</b></td><td>- pseudonim</td></tr>
<tr><td><b>%a</b></td><td>- wywietlany pseudonim</td></tr>
<tr><td><b>%f</b></td><td>- imi</td></tr>
<tr><td><b>%r</b></td><td>- nazwisko</td></tr>
<tr><td><b>%m</b></td><td>- numer telefonu</td></tr>
<tr><td><b>%u</b></td><td>- UIN</td></tr>
<tr><td><b>%g</b></td><td>- grupa</td></tr>
<tr><td><b>%o</b></td><td>- port (gdy za NAT'em)</td></tr>
<tr><td><b>%p</b></td><td>- port (z zewnetrznym IP)</td></tr>
<tr><td><b>%e</b></td><td>- e-mail</td></tr>
</table>
Ponadto, obsuguje ona wszystkie inne znaczniki dostpne w parserze HTML Kadu,
jak <b>{}</b>, <b>`'</b>, etc.
*/
int Tcl_CmdParse(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc != 3)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "parse UIN text");
        return TCL_ERROR;
    }
    UserListElement u;
    bool conv, ok = TRUE;
    QString altName = QString::fromUtf8(argv[1]);
    int uin = altName.toUInt(&conv);

    if (conv)
      {
        if (userlist.containsUin(uin))
        {
             u = userlist.byUin(uin);
        }
        else
        {
            ok = FALSE;
        }
    }
    else
    {
        if (userlist.containsAltNick(altName))
        {
             u = userlist.byAltNick(altName);
        }
        else
        {
            ok = FALSE;
        }
    }
    if (!ok)
    {
        QCString utf = QObject::tr("User %1 isn't registered on the list.").arg(altName).utf8();
        Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
        return TCL_ERROR;
    }
    QString ret = parse(QString::fromUtf8(argv[2]), u, FALSE);
    QCString utf = ret.utf8();
    Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
    Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
    return TCL_OK;
}

/* Tcl Command descriptions
SYNTAX: descriptions command ?arguments?
DESCRIPTION:
Komenda suca do manipulowania histori opisw Kadu. S to te opisy, ktre mona zobaczy,
gdy wybierze si status z opisem i pojawi si okno dialogowe, w ktrym wpisuje si swj opis.
Jest tam rozwijana lista, zawiera ona list opisw, do ktrych mona mie dostp za pomoc
tej komendy.

SUBSYNTAX: descriptions get ?numer?
DESCRIPTION:
Komenda <b>descriptsions get</b> pobiera wszystkie opisy i zwraca je w postaci
listy TCL, chyba e zostanie podany opcjonalny argument - liczba z zakresu 0-2, ktra oznacza,
ktry z kolei opis chcesz pobra.

SUBSYNTAX: descriptions set numer tekst
DESCRIPTION:
Komenda <b>descriptions set</b> pozwala na ustawianie tych opisw
wedug wasnych potrzeb. Wymagane s do niej 2 argumenty - liczba z zakresu 0-2, do okrelenia,
ktry wpis chce si zmieni, oraz tekst, ktry bdzie stanowi nowy opis.
*/
int Tcl_CmdDescriptions(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 4 || argc < 2)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "descriptions command ?arguments?");
        return TCL_ERROR;
    }
    QString cmd = argv[1];
    if (!cmd.compare("get"))
    {
        Tcl_Obj* result;
        QCString utf;
        if (argc > 3)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "descriptions get ?number?");
            return TCL_ERROR;
        }
        else if (argc == 3)
        {
            bool conv;
            int cnt = QString(argv[2]).toUInt(&conv);
            if (!conv)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "descriptions get ?number?");
                return TCL_ERROR;
            }
            if (cnt > 3)
            {
                QCString utf = QObject::tr("Kadu descriptions history supports only four descriptions. You ware traing to set description for %1 entry in history.").arg(argv[2]).utf8();
                Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                return TCL_ERROR;
            }
            utf = defaultdescriptions[cnt].utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
        }
        else
        {
            Tcl_DString ds;
            Tcl_DStringInit(&ds);
            for ( uint i = 0; i < defaultdescriptions.count(); i++ )
            {
                utf = defaultdescriptions[i].utf8();
                result = Tcl_NewStringObj((const char* const)utf, utf.length());
                Tcl_DStringAppendElement(&ds, Tcl_GetString(result));
            }
            Tcl_DStringResult(interp, &ds);
            Tcl_DStringFree(&ds);
        }
    }
    else if (!cmd.compare("set"))
    {
        if (argc != 4)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "descriptions set number text");
            return TCL_ERROR;
        }
        bool conv;
        int cnt = QString(argv[2]).toUInt(&conv);
        if (!conv)
        {
            Tcl_WrongNumArgs(interp, 0, NULL, "descriptions set number text");
            return TCL_ERROR;
        }
        if (cnt > 3)
        {
            QCString utf = QObject::tr("Kadu descriptions history supports only four descriptions. You ware traing to set description for %1 entry in history.").arg(argv[2]).utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
        }
        defaultdescriptions[cnt] = QString::fromUtf8(argv[3]);
    }
    else
    {
        QCString utf = QObject::tr("Command should be one of following: %1").arg("get, set").utf8();
        Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
        return TCL_ERROR;
    }
    return TCL_OK;
}


/* Tcl Command search
SYNTAX: search ?opcje?
DESCRIPTION:
Komenda suca gwnie przeszukiwaniu katalogu publicznego, ale moe te otworzy okno
dialogowe do tego suace, aby uytkownik zrobi to wasnorcznie. Operacje przeszukiwania
powinny opiera si dodatkowo na zdarzeniach <b>pubdir</b>, <b>pubdirnext</b>, oraz <b>searchdone</b>,
ktre posiadaj w argumentach identyfikator przeszukiwania, ten za jest zwracany przez komend search.
Jest to potrzebne, gdy chcemy okreli, czy dane zdarzenie (jedno z trzech ww.) jest wywokonywane wanie
dla naszego wywoania search.

SUBSYNTAX: search
DESCRIPTION:
Bez adnych argumentw otwiera po prostu znane z Kadu okno dialogowe do przeszukiwania katalogu
publicznego.

SUBSYNTAX: search -uin UIN
DESCRIPTION:
Szuka w katalogu publicznym podanego UIN'u. Zwraca identyfikator przeszukiwania.

SUBSYNTAX: search -nick pseudonim -first imi -last nazwisko -city miasto -born rok_urodzenia -sex pe -from pozycja -available
DESCRIPTION:
Najczciej spotykana posta przeszkukiwania katalogu publicznego. Dane personalne poszukiwanej osoby
podaje si jako argumenty odpowiednich opcji. Na uwag zasuguj 4 opcje:<br>
<b>-born</b> - naley poda rok urodzenia w formie 19xx (lub 20xx)<br>
<b>-sex</b> - pe mona okreli na kilka sposobw: <b>m</b>, <b>male</b>, <b>man</b> - oznacza mczyzne,
natomiast <b>f</b>, <b>female</b>, <b>woman</b> - oznacza kobiet.
<b>-from</b> - jest to pozycja od ktrej naley rozpocz przeszukiwanie. Opcja ta jest zazwyczaj uywana
jedynie w przypadku, gdy liczba odpowiedzi pasujcych do zapytania przekracza 20. Wtedy wywoywane jest
zdarzenie <b>pubdirnext</b> ktre zawiera pozycj, od ktrej naley rozpocz dalsze poszukiwania tak,
aby nie powtrzyy si te odpowiedzi, ktre ju mamy. <b>UWAGA!</b> Naley uwaa z automatyzacj
przeszukiwania katalogu a do znalezienia wszystkich moliwych odpowiedzi, gdy w niektrych moe
ich by na prawd duo i serwer GG moe zablokowa dostp do katalogu publicznego naszemu numerowi
na jaki czas.<br>
<b>-available</b> - jedyna opcja nie wymagajca argumentu. Jej uycie powoduje otrzymanie odpowiedzi
jedynie o kontaktach koystajcych w tym momencie z GG.<br><br>
Jedno z dwch zdarze musi by zwrcone po wywoaniu komendy <b>search</b> (prcz zdarzenia <b>pubdir</b>):
<b>pubdirnext</b> lub <b>searchdone</b>.

EXAMPLE:
set mojID ""
proc a {id uin nick name born city status} {
    global mojID
    if {$mojID == $id} {
        puts "Znaleziono: <$uin> Imi $name, ksywka $nick, miasto $city."
    }
}
proc b {id position} {
    global mojID
    if {$mojID == $id} {
        set mojID [search -born 1984 -available -from $position]
    }
}
proc c {id} {
    global mojID
    if {$mojID == $id} {
        puts "Koniec szukania."
        set mojID ""
    }
}

on pubdir 1 a
on pubdirnext 1 b
on searchdone 1 c

set mojID [search -born 1984 -available]
ENDOFEXAMPLE
*/
int Tcl_CmdSearch(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc > 16)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "search ?options?");
        return TCL_ERROR;
    }
    QStringList options = QStringList::split(" ", "-nick -last -first -city -born -sex -from -available");
    if (argc > 1)
    {
        if (getCurrentStatus() == GG_STATUS_NOT_AVAIL)
        {
            QCString utf = QObject::tr("Cannot search in public directory while you're offline.").utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
        }
        int from = 0;
        gg_pubdir50_t req = gg_pubdir50_new(GG_PUBDIR50_SEARCH);
        if (!QString::compare(argv[1], "-uin"))
        {
            if (argc != 3)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "search -uin UIN");
                return TCL_ERROR;
            }
            QString uin = argv[2];
            bool conv;
            uin.toUInt(&conv);
            if (!conv)
            {
                Tcl_WrongNumArgs(interp, 0, NULL, "search -uin UIN");
                return TCL_ERROR;
            }
            gg_pubdir50_add(req, GG_PUBDIR50_UIN, (const char *)unicode2cp(uin).data());
        }
        else
        {
            int idx;
            char *type;
            QString arg;
            for ( int i = 1; i < argc; i++ )
            {
                idx = options.findIndex(argv[i]);
                if (idx < 7 && idx > -1)
                    i++;

                switch (idx)
                {
                    case 0:
                        type = GG_PUBDIR50_NICKNAME;
                        arg = QString::fromUtf8(argv[i]);
                        break;
                    case 1:
                        type = GG_PUBDIR50_LASTNAME;
                        arg = QString::fromUtf8(argv[i]);
                        break;
                    case 2:
                        type = GG_PUBDIR50_FIRSTNAME;
                        arg = QString::fromUtf8(argv[i]);
                        break;
                    case 3:
                        type = GG_PUBDIR50_CITY;
                        arg = QString::fromUtf8(argv[i]);
                        break;
                    case 4:
                        type = GG_PUBDIR50_BIRTHYEAR;
                        arg = QString("%1 %1").arg(QString::fromUtf8(argv[i]));
                        break;
                    case 5:
                    {
                        QStringList male = QStringList::split(" ", "m male man"),
                                    female =  QStringList::split(" ", "f female woman");
                        if (male.findIndex(arg) > -1)
                        {
                            gg_pubdir50_add(req, GG_PUBDIR50_GENDER, GG_PUBDIR50_GENDER_MALE);
                            continue;
                        }
                        else if (female.findIndex(arg) > -1)
                        {
                            gg_pubdir50_add(req, GG_PUBDIR50_GENDER, GG_PUBDIR50_GENDER_FEMALE);
                            continue;
                        }
                        else
                        {
                            QCString utf = QObject::tr("Unknown sex: %1").arg(arg).utf8();
                            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                            return TCL_ERROR;
                        }
                        break;
                    }
                    case 6:
                    {
                        bool conv;
                        from = QString::fromUtf8(argv[i]).toInt(&conv);
                        if (!conv)
                        {
                            QCString utf = QObject::tr("%1 is not a number. Option -from needs the number as an argument.").arg(arg).utf8();
                            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                            return TCL_ERROR;
                        }
                        continue;
                    }
                    case 7:
                    {
                        gg_pubdir50_add(req, GG_PUBDIR50_ACTIVE, GG_PUBDIR50_ACTIVE_TRUE);
                        continue;
                    }
                    default:
                    {
                        QCString utf = QObject::tr("Option should be one of following: %1").arg(options.join(", ")).utf8();
                        Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
                        Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
                        return TCL_ERROR;
                    }
                }
                gg_pubdir50_add(req, type, (const char *)unicode2cp(arg).data());
            }
        }
        gg_pubdir50_add(req, GG_PUBDIR50_START, QString::number(from).local8Bit());
        int seq = gg_pubdir50(sess, req);
        gg_pubdir50_free(req);
        tclPubDirList[seq] = from;
        Tcl_SetResult(interp, (char*)QString::number(seq).latin1(), TCL_VOLATILE);
        return TCL_OK;
    }
    else
    {
        SearchDialog* win = new SearchDialog();
        win->show();
        return TCL_OK;
    }
}

#ifdef USE_TK
/* Tcl Command GUI
SYNTAX: GUI
DESCRIPTION:
Uruchamia TCL'owskiego menadera GUI - TK dla bierzcej sesji TCL.
Automatycznie pojawia si pierwsze okno, gwne, ktre automatycznie
zostaje natychmiast schowane. Daje to efekt mignicia maego okna na
ekranie. Nie naley si tym przejmowa. To normalne :) Wszelkie
interfejsy nalezy budowa na oknach podrzdnych okna gownego, tj.
".mojeGui", czy ".jakiesOkno", a dopiero w nich odpowiednie kontrolki.
*/
int Tcl_CmdGui(void* clientData, Tcl_Interp *interp, int argc, const char *argv[])
{
    if (argc != 1)
    {
        Tcl_WrongNumArgs(interp, 0, NULL, "GUI");
        return TCL_ERROR;
    }
    int sessID = *((int*) clientData);
    TclSession *tsess = tclSess[sessID];
    if (!tsess->isGui())
    {
        if (Tk_Init(interp) == TCL_ERROR)
        {
            QCString utf = QObject::tr("Error while initializing TK intepreter:\n%1").arg(Tcl_GetStringResult(interp)).utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
            return TCL_ERROR;
        }
        tsess->setGui();
        tsess->hideMainWindow();
    }
    return TCL_OK;
}

#endif

// Czesc klasy TclSession
void TclSession::addTclCommands()
{
    Tcl_CreateCommand(tclInt(), "chat", Tcl_CmdChat, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "configfile", Tcl_CmdConfigFile, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "hint", Tcl_CmdHint, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "ignore", Tcl_CmdIgnore, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "on", Tcl_CmdEvent, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "msg", Tcl_CmdMsg, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "play", Tcl_CmdPlay, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "quit", Tcl_CmdQuit, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "timer", Tcl_CmdTimer, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "user", Tcl_CmdUser, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "status", Tcl_CmdStatus, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "connect", Tcl_CmdConnect, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "disconnect", Tcl_CmdDisconnect, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "console", Tcl_CmdConsole, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "config", Tcl_CmdConfig, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "usermenu", Tcl_CmdUserMenu, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "trayicon", Tcl_CmdTrayIcon, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "parse", Tcl_CmdParse, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "descriptions", Tcl_CmdDescriptions, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "search", Tcl_CmdSearch, idPtr, NULL);

#ifdef USE_TK
    Tcl_CreateCommand(tclInt(), "GUI", Tcl_CmdGui, idPtr, NULL);
#endif

    // XMMS module
//    Tcl_CreateCommand(tclInt(), "xmms", Tcl_CmdXmms, idPtr, NULL);

    Tcl_CreateCommand(tclInt(), "popup", Tcl_CmdPopup, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "inputdialog", Tcl_CmdInputDialog, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "filedialog", Tcl_CmdFileDialog, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "msgbox", Tcl_CmdMessageBox, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "qlistbox", Tcl_CmdListBox, idPtr, NULL);
    Tcl_CreateCommand(tclInt(), "qmenu", Tcl_CmdMenu, idPtr, NULL);
}
