/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include "tcl_scripting.h"

#include "events.h"

TclEventHandler::TclEventHandler(TclSession* s)
{
    kdebugf();
    tsess = s;

    connect(&event_manager, SIGNAL(connected()), this, SLOT(connected()));
    connect(&event_manager, SIGNAL(connectionFailed(int)), this, SLOT(connectionFailed(int)));
    connect(&event_manager, SIGNAL(connectionBroken()), this, SLOT(connectionBroken()));
    connect(&event_manager, SIGNAL(disconnected()), this, SLOT(disconnected()));
    connect(&event_manager, SIGNAL(userStatusChanged(struct gg_event*)), this, SLOT(userStatusChanged(struct gg_event*)));
    connect(&event_manager, SIGNAL(userlistReceived(struct gg_event*)), this, SLOT(userlistReceived(struct gg_event*)));
    connect(&event_manager, SIGNAL(messageReceived(int, UinsList, QCString&, time_t, QByteArray&)), this, SLOT(messageReceived(int, UinsList, QCString&, time_t, QByteArray&)));
    connect(&event_manager, SIGNAL(chatMsgReceived0(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived0(UinsList, const QString&, time_t, bool&)));
    connect(&event_manager, SIGNAL(chatMsgReceived1(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived1(UinsList, const QString&, time_t, bool&)));
    connect(&event_manager, SIGNAL(chatMsgReceived2(UinsList, const QString&, time_t)), this, SLOT(chatReceived2(UinsList, const QString&, time_t)));
    connect(&event_manager, SIGNAL(userlistReplyReceived(char, char *)), this, SLOT(userlistReplyReceived(char, char *)));
    connect(&event_manager, SIGNAL(ackReceived(int)), this, SLOT(ackReceived(int)));
    connect(&event_manager, SIGNAL(pubdirReplyReceived(gg_pubdir50_t)), this, SLOT(pubdirReplyReceived(gg_pubdir50_t)));
    connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
    connect(chat_manager, SIGNAL(chatDestroyed(const UinsList&)), this, SLOT(chatDestroyed(const UinsList&)));
    connect(&userlist, SIGNAL(changingStatus(const UinType, const unsigned int, const unsigned int)), this, SLOT(userStatusChanging(const UinType, const unsigned int, const unsigned int)));
    connect(&userlist, SIGNAL(modified()), this, SLOT(userlist_modified()));
    connect(&userlist, SIGNAL(userAdded(const UserListElement&)), this, SLOT(userlist_userAdded(const UserListElement&)));
    connect(&userlist, SIGNAL(dnsNameReady(UinType)), this, SLOT(userlist_dnsNameReady(UinType)));
    connect(kadu, SIGNAL(changingStatus(int, QString, bool&)), this, SLOT(changingStatus(int, QString, bool&)));
    connect(this, SIGNAL(searchingFinished(int)), this, SLOT(searchInPubDirFinished(int)));

    connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated_Watch(const UinsList&)));

    for ( uint i = 0; chat_manager->chats().count() > i; i++ )
        manageReadyChats(chat_manager->chats()[i], TRUE);
}

TclEventHandler::~TclEventHandler()
{
    kdebugf();
    disconnect(&event_manager, SIGNAL(connected()), this, SLOT(connected()));
    disconnect(&event_manager, SIGNAL(connectionFailed(int)), this, SLOT(connectionFailed(int)));
    disconnect(&event_manager, SIGNAL(connectionBroken()), this, SLOT(connectionBroken()));
    disconnect(&event_manager, SIGNAL(disconnected()), this, SLOT(disconnected()));
    disconnect(&event_manager, SIGNAL(userStatusChanged(struct gg_event*)), this, SLOT(userStatusChanged(struct gg_event*)));
    disconnect(&event_manager, SIGNAL(userlistReceived(struct gg_event*)), this, SLOT(userlistReceived(struct gg_event*)));
    disconnect(&event_manager, SIGNAL(messageReceived(int, UinsList, QCString&, time_t, QByteArray&)), this, SLOT(messageReceived(int, UinsList, QCString&, time_t, QByteArray&)));
    disconnect(&event_manager, SIGNAL(chatMsgReceived0(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived0(UinsList, const QString&, time_t, bool&)));
    disconnect(&event_manager, SIGNAL(chatMsgReceived1(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived1(UinsList, const QString&, time_t, bool&)));
    disconnect(&event_manager, SIGNAL(chatMsgReceived2(UinsList, const QString&, time_t)), this, SLOT(chatReceived2(UinsList, const QString&, time_t)));
    disconnect(&event_manager, SIGNAL(userlistReplyReceived(char, char *)), this, SLOT(userlistReplyReceived(char, char *)));
    disconnect(&event_manager, SIGNAL(ackReceived(int)), this, SLOT(ackReceived(int)));
    disconnect(&event_manager, SIGNAL(pubdirReplyReceived(gg_pubdir50_t)), this, SLOT(pubdirReplyReceived(gg_pubdir50_t)));
    disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
    disconnect(chat_manager, SIGNAL(chatDestroyed(const UinsList&)), this, SLOT(chatDestroyed(const UinsList&)));
    disconnect(&userlist, SIGNAL(changingStatus(const UinType, const unsigned int, const unsigned int)), this, SLOT(userStatusChanging(const UinType, const unsigned int, const unsigned int)));
    disconnect(&userlist, SIGNAL(modified()), this, SLOT(userlist_modified()));
    disconnect(&userlist, SIGNAL(userAdded(const UserListElement&)), this, SLOT(userlist_userAdded(const UserListElement&)));
    disconnect(&userlist, SIGNAL(dnsNameReady(UinType)), this, SLOT(userlist_dnsNameReady(UinType)));
    disconnect(kadu, SIGNAL(changingStatus(int, QString, bool&)), this, SLOT(changingStatus(int, QString, bool&)));
    disconnect(this, SIGNAL(searchingFinished(int)), this, SLOT(searchInPubDirFinished(int)));

    disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated_Watch(const UinsList&)));

    for ( uint i = 0; chat_manager->chats().count() > i; i++ )
        manageReadyChats(chat_manager->chats()[i], FALSE);
}

void TclEventHandler::chatCreated_Watch(const UinsList& senders)
{
    kdebugf();
    Chat* chat = chat_manager->findChatByUins(senders);
    connect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(chatMsgSendReq(Chat*)));
    connect(chat, SIGNAL(messageSent(Chat*)), this, SLOT(chatMsgSent(Chat*)));
    connect(chat, SIGNAL(messageFiltering(const UinsList&, QCString&, bool&)), this, SLOT(outputMessageFiltering(const UinsList&, QCString&, bool&)));
    connect(chat->edit, SIGNAL(keyPressed(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyPressed(QKeyEvent*, CustomInput*)));
}

void TclEventHandler::manageReadyChats(Chat* chat, bool conn)
{
    kdebugf();
    if (conn)
    {
        connect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(chatMsgSendReq(Chat*)));
        connect(chat, SIGNAL(messageSent(Chat*)), this, SLOT(chatMsgSent(Chat*)));
        connect(chat, SIGNAL(messageFiltering(const UinsList&, QCString&, bool&)), this, SLOT(outputMessageFiltering(const UinsList&, QCString&, bool&)));
        connect(chat->edit, SIGNAL(keyPressed(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyPressed(QKeyEvent*, CustomInput*)));
    }
    else
    {
        disconnect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(chatMsgSendReq(Chat*)));
        disconnect(chat, SIGNAL(messageSent(Chat*)), this, SLOT(chatMsgSent(Chat*)));
        disconnect(chat, SIGNAL(messageFiltering(const UinsList&, QCString&, bool&)), this, SLOT(outputMessageFiltering(const UinsList&, QCString&, bool&)));
        disconnect(chat->edit, SIGNAL(keyPressed(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyPressed(QKeyEvent*, CustomInput*)));
    }
}

/* Tcl Event
ack identyfikator_wiadomoci[liczba]
Otrzymano potwierdzenie wiadomoci.
*/
void TclEventHandler::ackReceived(int seq)
{
    kdebug("TCL Event: ackReceived(%d)\n", seq);
    int ret;
    QString filter, code, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("ack"))
        {
            code = it.data() + " " + QString::number(seq);
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
chat0 wysyajce_UINy[lista_liczb] tre[acuch] czas[unixtime]
Otrzymano wiadomo klasy chat, ktra jeszcze nie zostaa wywietlona w oknie Chat,
ani dodana do historii. Jeli funkcja obsugujca to zdarzenie zwrci warto logiczn 1,
to zdarzenia chat1 ani chat2 nie nastpi, a uytkownik nie zostanie powiadomiony
o nowej wiadomoci, ani te jej nie zobaczy.
*/
void TclEventHandler::chatReceived0(UinsList senders, const QString& msg, time_t time, bool& grab)
{
    kdebug("TCL Event: chatReceived0(%s)\n", msg.latin1());
    int ret;
    QString filter, code, key, err;
    QStringList uins;

    QValueList<uin_t>::Iterator _it;
    for( _it = senders.begin(); _it != senders.end(); _it++ )
        uins.append(QString::number(*_it));

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("chat0"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, uins.join(" ").latin1());

            QCString utf = msg.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            Tcl_DStringAppendElement(&ds, QString::number(time));
            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            QString tmp = Tcl_GetStringResult(tsess->tclInt());
            if (!tmp.isEmpty())
            {
                int retVal;
                Tcl_ResetResult(tsess->tclInt());
                ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
                if (ret != TCL_ERROR)
                    if (retVal)
                        grab = TRUE;
            }
            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
chat1 wysyajce_UINy[lista_liczb] tre[acuch] czas[unixtime]
Otrzymano wiadomo klasy chat, ktra jeszcze nie zostaa wysana do uytkownika Kadu.
Jeli funkcja obsugujca to zdarzenie zwrci warto logiczn 1,
to zdarzenie chat2 nie nastpi, a uytkownik nie zostanie powiadomiony o nowej wiadomoci.
*/
void TclEventHandler::chatReceived1(UinsList senders, const QString& msg, time_t time, bool& grab)
{
    kdebug("TCL Event: chatReceived1(%s)\n", msg.latin1());
    int ret;
    QString filter, code, key, err;
    QStringList uins;

    QValueList<uin_t>::Iterator _it;
    for( _it = senders.begin(); _it != senders.end(); _it++ )
        uins.append(QString::number(*_it));

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("chat1"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, uins.join(" ").latin1());

            QCString utf = msg.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            Tcl_DStringAppendElement(&ds, QString::number(time));
            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            QString tmp = Tcl_GetStringResult(tsess->tclInt());
            if (!tmp.isEmpty())
            {
                int retVal;
                Tcl_ResetResult(tsess->tclInt());
                ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
                if (!chat_manager->findChatByUins(senders))
                {
                    if (ret != TCL_ERROR)
                        if (retVal)
                            grab = TRUE;
                }
            }
            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
chat2 wysyajce_UINy[lista_liczb] tre[acuch] czas[unixtime]
Otrzymano wiadomo klasy chat, o ktrej uytkownik zosta ju powiadomiony.
*/
void TclEventHandler::chatReceived2(UinsList senders, const QString& msg, time_t time)
{
    kdebug("TCL Event: chatReceived2(%s)\n", msg.latin1());
    int ret;
    QString filter, code, key;
    QStringList uins;

    QValueList<uin_t>::Iterator _it;
    for( _it = senders.begin(); _it != senders.end(); _it++ )
        uins.append(QString::number(*_it));

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("chat2"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, uins.join(" ").latin1());

            QCString utf = msg.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            Tcl_DStringAppendElement(&ds, QString::number(time));
            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
chatkey UINy_okna_chat[lista_liczb] klawisz[acuch] modyfikator[acuch] kod_ASCII[liczba]
Wcinito pewien klawisz w oknie chat, ktrego UINy s dane. Jeli wcinity klawisz
jest znakiem, ktry mona wywietli, to argument klawisza bdzie zawiera ten znak,
natomiast jeli jest to klawisz specjalny, np. Home, Shift, czy Strzalka w lewo, wtedy
argument klawisza bedzie zawiera odpowiednio: home, shift, left. Pozostae klawisze
analogicznie. Argument modyfikatora zawiera jeden z trzech modyfikatorw
(<b>alt</b>, <b>control</b>, <b>shift</b>) jeli podczas wcinicia aktualnego klawisza ktry
z tych modyfikatorw by rwnierz trzymany. Niemoliwe jest wystpienie wicej ni jednego
modyfikatora. Jeli dany klawisz nie jest rozpoznawany, to zwrcone zostanie sowo <b>unknown</b>.
Sowo to jest rwnierz zwracane dla klawiszy wciskanych razem z z modyfikatorem <b>control</b>.
W takim przypadku, aby rozezna si, ktry klawisz zosta wciniety, naley posuy si
ostatnim z argumentw - kodem ASCII znaku otrzymanego przez kombinacj <b>control</b> + klawisz.
Przykadowo, dla kombinacji <b>ctrl+a</b> byaby to liczba 1, dla <b>ctrl+d</b> 4, itd.
*/
void TclEventHandler::chatKeyPressed(QKeyEvent* e, CustomInput* k)
{
    int ret;
    QString filter, code, key, keystr = e->text(), modstr = "", ascii = QString::number(e->ascii());
    Chat* chat = (Chat*)k->parentWidget()->parentWidget()->parentWidget();
    UinsList senders = chat->uins();

    TclMisc::getKey(e, keystr, modstr);

    QStringList uins;
    QValueList<uin_t>::Iterator _it;
    for( _it = senders.begin(); _it != senders.end(); _it++ )
        uins.append(QString::number(*_it));

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("chatkey"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, uins.join(" ").latin1());
            Tcl_DStringAppendElement(&ds, keystr.latin1());
            Tcl_DStringAppendElement(&ds, modstr.latin1());
            Tcl_DStringAppendElement(&ds, ascii.latin1());

            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
chmystatus status[acuch] opis[acuch]
Uytkownik Kadu zmienia status. Jeli funkcja obsugujca to zdarzenie
zwrci warto 1, to status ten nie zostanie ustawiony mimo wszystko.
*/
void TclEventHandler::changingStatus(int status, QString sigDesc, bool& stop)
{
    kdebug("TCL Event: changingStatus(%d, %s, %d)\n", status, sigDesc.latin1(), stop);
    int ret;
    QString filter, code, key, err,
        stat = TclMisc::GGStatusToTclStatus(status);

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("chmystatus"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, stat.latin1());

            QCString utf = sigDesc.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            QString tmp = Tcl_GetStringResult(tsess->tclInt());
            if (!tmp.isEmpty())
            {
                int retVal;
                Tcl_ResetResult(tsess->tclInt());
                ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
                if (ret != TCL_ERROR)
                    if (retVal)
                        stop = TRUE;
            }
            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
chstatus0 uin[liczba] stary_status[acuch] nowy_status[acuch]
Ktry z kontaktw zmienia swj status. To zdarzenie wystpuje
tu przed <b>chstatus1</b> i pozwala okreli jaki status mia
dany kontakt wczeniej (przed zmian). Aby uzyska opis statusu
przed zmian, naley uy komend <b>user get</b> <i>UIN</i> <b>-description</b>.
*/
void TclEventHandler::userStatusChanging(const UinType uin, const unsigned int oldstatus, const unsigned int status)
{
    kdebug("TCL Event: userStatusChanging(%d, %d, %d)\n", uin, oldstatus, status);
    int ret;
    QString filter, code, key,
        old_status = TclMisc::GGStatusToTclStatus(oldstatus),
        new_status = TclMisc::GGStatusToTclStatus(status);

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("chstatus0"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, QString::number(uin));
            Tcl_DStringAppendElement(&ds, old_status.latin1());
            Tcl_DStringAppendElement(&ds, new_status.latin1());
            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
chstatus1 uin[liczba] status[acuch] opis[acuch] adres[ip] port[liczba]
Ktry z kontaktw zmieni swj status.
*/
void TclEventHandler::userStatusChanged(struct gg_event* e)
{
    int status, ret;
    uint32_t uin;
    char *descr;
    uint32_t remote_ip;
    uint16_t remote_port;
 
    if (e->type == GG_EVENT_STATUS60)
    {
        uin = e->event.status60.uin;
        status = e->event.status60.status;
        descr = e->event.status60.descr;
        remote_ip = e->event.status60.remote_ip;
        remote_port = e->event.status60.remote_port;
    }
    else
    {
        uin = e->event.status.uin;
        status = e->event.status.status;
        descr = e->event.status.descr;
        remote_ip = 0;
        remote_port = 0;
    }
    
    QString filter, code, _status, key;

    kdebug("TCL Event: userStatusChanged(%d, %d, %s, %d, %d)\n", uin, status, descr, remote_ip, remote_port);

    _status = TclMisc::GGStatusToTclStatus(status);
    if (_status.isEmpty())
        tsess->error(tr("Unknown status type %1 in event chstatus.").arg(QString::number(status)));

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("chstatus"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, (char*)QString::number(uin).latin1());
            Tcl_DStringAppendElement(&ds, _status);

            QCString utf = cp2unicode((unsigned char *)descr).utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            Tcl_DStringAppendElement(&ds, (char*)QString::number(remote_ip).latin1());
            Tcl_DStringAppendElement(&ds, (char*)QString::number(remote_port).latin1());
            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
closechat UINy[lista_liczb]
Zostao zaknite okno rozmowy.
*/
void TclEventHandler::chatDestroyed(const UinsList& senders)
{
    int ret;
    QStringList uins;
    QString filter, code, key;
    for( uint i = 0; senders.count() > i; i++ )
        uins.append(QString::number(senders[i]));

    kdebug("TCL Event: chatDestroyed(%s)\n", uins.join(" ").latin1());
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("closechat"))
        {
            code = it.data() + " \{" + uins.join(" ") + "}";
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
connect
Nawizano poczenie z serwerem.
*/
void TclEventHandler::connected()
{
    kdebug("TCL Event: connected()\n");
    QString filter, key;
    int ret;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("connect"))
        {
            ret = Tcl_Eval(tsess->tclInt(), it.data().latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
connfail identyfikator_bdu[liczba]
Bd poczenia z serwerem.
*/
void TclEventHandler::connectionFailed(int why)
{
    kdebug("TCL Event: connectionFailed(%d)\n", why);
    int ret;
    QString filter, code, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("connfail"))
        {
            code = it.data() + " " + QString::number(why);
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
connbroke
Poczenia z serwerem zostao przerwane.
*/
void TclEventHandler::connectionBroken()
{
    kdebug("TCL Event: connectionBroken()\n");
    int ret;
    QString filter, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("connbroke"))
        {
            ret = Tcl_Eval(tsess->tclInt(), it.data().latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
dcc wysyajcy_UIN[liczba]
Otrzymano DCC CTCP.
*/
void TclEventHandler::dccConnectionReceived(const UserListElement& sender)
{
    kdebug("TCL Event: dccConnectionReceived(%d)\n", sender.uin);
    int ret;
    QString filter, code, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("dcc"))
        {
            code = it.data() + " " + QString::number(sender.uin);
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
disconnected
Otrzymano polecenie rozczenia.
*/
void TclEventHandler::disconnected()
{
    kdebug("TCL Event: disconnected()\n");
    int ret;
    QString filter, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("disconnect"))
        {
            ret = Tcl_Eval(tsess->tclInt(), it.data().latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
listreply typ[znak] odpowied[acuch]
Otrzymano z serwera rezultat operacji na licie uytkownikw.
*/
void TclEventHandler::userlistReplyReceived(char type, char *reply)
{
    kdebug("TCL Event: userlistReplyReceived(%c, %s)\n", type, reply);
    int ret;
    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QString filter, code, key;
    QMap<QString,QString>::Iterator it;
    for ( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("listreply"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, &type);

            QCString utf = cp2unicode((unsigned char *)reply).utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
msg typ[liczba] wysyajce_UINy[lista_liczb] tre[acuch] czas[unixtime]
Otrzymano jak wiadomo z serwera GG.
*/
void TclEventHandler::messageReceived(int msgclass, UinsList senders, QCString &msg,
                                      time_t time, QByteArray &formats)
{
    kdebug("TCL Event: messageReceived(%d, %s)\n", msgclass, (const char*)msg);
    int ret;
    const char* msg_c = msg;
    QString filter, code, key, _msg = cp2unicode((const unsigned char*)msg_c);
    QStringList uins;

    QValueList<uin_t>::Iterator _it;
    for( _it = senders.begin(); _it != senders.end(); _it++ )
        uins.append(QString::number(*_it));

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("msg"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, (char*)QString::number(msgclass).latin1());
            Tcl_DStringAppendElement(&ds, uins.join(" ").latin1());
            
            QCString utf = _msg.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            Tcl_DStringAppendElement(&ds, (char*)QString::number(time).latin1());
            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
msgsent UINy[lista_liczb]
Wiadomo z okna rozmowy z tym(i) UINem(ami) zosta ju wysana,
ale nie otrzymano jeszcze potwierdzenia, po prostu poszo w wiat.
*/
void TclEventHandler::chatMsgSent(Chat* chat)
{
    int ret;
    UinsList senders = chat->uins();
    QStringList uins;
    QString filter, code, key;
    for( uint i = 0; senders.count() > i; i++ )
        uins.append(QString::number(senders[i]));

    kdebug("TCL Event: chatMsgSendReq(%s)\n", uins.join(" ").latin1());
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("msgsent"))
        {
            code = it.data() + " \{" + uins.join(" ") + "}";
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
newchat UINy[lista_liczb]
Zostao otwarte nowe okno rozmowy.
*/
void TclEventHandler::chatCreated(const UinsList& senders)
{
    int ret;
    QStringList uins;
    QString filter, code, key;
    for( uint i = 0; senders.count() > i; i++ )
        uins.append(QString::number(senders[i]));

    kdebug("TCL Event: chatCreated(%s)\n", uins.join(" ").latin1());
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("newchat"))
        {
            code = it.data() + " \{" + uins.join(" ") + "}";
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
progresscancel
Anulowano okno postpu.
*/
void TclEventHandler::msgBoxCancel()
{
    kdebug("TCL Event: msgBoxCancel()\n");
    int ret;
    QString filter, code, key;
    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("progresscancel"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
pubdir ID[liczba] szukany(a)_UIN[liczba] pseudonim[acuch] imi[acuch] urodziny[data] miasto[acuch] status[acuch]
Otrzymano z serwera rezultat operacji wyszukiwania w katalogu publicznym. <i>ID</i> jest identyfikatorem
przeszukiwania katalogu publicznego. Komenda <b>search</b> zwraca taki identyfikator w celu dopasowania,
czy otrzymane rezultaty wyszukiwania s tymi, o ktre pytana ww. komend.
*/
void TclEventHandler::pubdirReplyReceived(gg_pubdir50_t res)
{
    kdebug("TCL Event: pubdirReplyReceived()\n");
    char *uin, *first, *nick, *born, *city, *status;
    QString filter, code, __status, key, id;
    int _status;

    int count = gg_pubdir50_count(res), ret;
    if (count > 0)
    {
        for (int i = 0; i < count; i++)
        {
            uin = (char*)gg_pubdir50_get(res, i, GG_PUBDIR50_UIN);
            first = (char*)gg_pubdir50_get(res, i, GG_PUBDIR50_FIRSTNAME);
            nick = (char*)gg_pubdir50_get(res, i, GG_PUBDIR50_NICKNAME);
            born = (char*)gg_pubdir50_get(res, i, GG_PUBDIR50_BIRTHYEAR);
            city = (char*)gg_pubdir50_get(res, i, GG_PUBDIR50_CITY);
            status = (char*)gg_pubdir50_get(res, i, GG_PUBDIR50_STATUS);

            if (status == NULL)
                return;

            _status = atoi(status) & 127;
            __status = TclMisc::GGStatusToTclStatus(_status);
            if (__status.isEmpty())
                  tsess->error(tr("Status type in event 'pubdirReplyReceived' is unknown."));

            Tcl_DString ds;
            Tcl_DStringInit(&ds);
            QMap<QString,QString>::Iterator it;
            for ( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
            {
                key = QStringList::split(":", it.key()).first();
                filter = key;
                if (!filter.compare("pubdir"))
                {
                    Tcl_DStringAppendElement(&ds, it.data().latin1());
                    Tcl_DStringAppendElement(&ds, QString::number(res->seq).latin1());
                    Tcl_DStringAppendElement(&ds, uin);

                    QCString utf;
                    Tcl_Obj* result;

                    utf = cp2unicode((unsigned char *)nick).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

                    utf = cp2unicode((unsigned char *)first).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

                    utf = cp2unicode((unsigned char *)born).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

                    utf = cp2unicode((unsigned char *)city).utf8();
                    result = Tcl_NewStringObj((const char* const)utf, utf.length());
                    Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

                    Tcl_DStringAppendElement(&ds, __status.latin1());
                    ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
                    Tcl_DStringFree(&ds);
                    if (ret == TCL_ERROR)
                        tsess->error(Tcl_GetStringResult(tsess->tclInt()));

                    Tcl_ResetResult(tsess->tclInt());
                }
            }
        }
    }
    checkForNext(res, count);
}

/* Tcl Event
pubdirnext ID[liczba] pozycja[liczba]
To zdarzenie podaje numer <i>pozycji</i>, od ktrej naley rozpocz dalsze przeszukiwanie (o ile potrzeba)
katalogu publicznego dla podanego <i>identyfikatora</i> szukania (zwrconego przez komend <b>search</b>).
Wystpuje ono jedynie wtedy, gdy zwrcone wyniki nie s wszystkimi moliwymi (w katalogu jest wicej ni 20
pasujcych odpowiedzi). Numer pozycji jest argumentem dla opcji <b>-from</b> komendy <b>search</b>.
Jeli ju wszystkie odpowiedzi zostay odebrane, to <i>pozycja</i> bdzie wynosi 0.
*/
void TclEventHandler::checkForNext(gg_pubdir50_t res, int count)
{
    int ret;
    QString filter, code, key;
    if (tclPubDirList.keys().findIndex(res->seq) == -1)
        return;

    if (count < 1)
    {
        tclPubDirList.remove(res->seq);
        emit searchingFinished(res->seq);
        return;
    }
    int next = gg_pubdir50_next(res);
    if (next == tclPubDirList[res->seq])
    {
        tclPubDirList.remove(res->seq);
        emit searchingFinished(res->seq);
        return;
    }
    tclPubDirList.remove(res->seq);

    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("pubdirnext"))
        {
            code = it.data() + " " + QString::number(res->seq) + " " + QString::number(next);
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
searchdone ID[liczba]
Zakoczono wyszukiwanie dla danego <i>identyfikatora</i> zwrconego przez komend <b>search</b>.
*/
void TclEventHandler::searchInPubDirFinished(int id)
{
    kdebug("TCL Event: searchingFinished(%d))\n", id);
    int ret;
    QString filter, code, key;

    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("searchdone"))
        {
            code = it.data() + " " + QString::number(id);
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
sendmsg1 UINy[lista_liczb]
Uytkownik wcisn Enter, rzdajc tym samym wysania wiadomoci do
UINa(ow). Jest to odpowiedni moment na edycj tekstu, przed wysaniem do serwera.
*/
void TclEventHandler::chatMsgSendReq(Chat* chat)
{
    int ret;
    UinsList senders = chat->uins();
    QStringList uins;
    QString filter, code, key;
    for( uint i = 0; senders.count() > i; i++ )
        uins.append(QString::number(senders[i]));

    kdebug("TCL Event: chatMsgSendReq(%s)\n", uins.join(" ").latin1());
    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("sendmsg1"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());

            QCString utf;
            Tcl_Obj* result;

            utf = uins.join(" ").utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
sendmsg2 UINy[lista_liczb] wiadomo[acuch]
Zdarzenie wywolywane tuz przed wysaniem, <i>wiadomoci</i> do <i>UINw</i>, ju w jej docelowej
postaci. Jeli procedura obsugujca to zdarzenie zwrci warto 1, to wiadomo nie zostanie
ostatecznie wysana.
*/
void TclEventHandler::outputMessageFiltering(const UinsList& senders, QCString& msg, bool &stop)
{
    kdebug("TCL Event: outputMessageFiltering(%s)\n", (const char*)msg);
    int ret;
    const char* msg_c = msg;
    QString filter, code, key, err, _msg = cp2unicode((const unsigned char*)msg_c);
    QStringList uins;

    for( uint i = 0; senders.count() > i; i++ )
        uins.append(QString::number(senders[i]));

    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("sendmsg2"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());
            Tcl_DStringAppendElement(&ds, uins.join(" ").latin1());

            QCString utf = _msg.utf8();
            Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            QString tmp = Tcl_GetStringResult(tsess->tclInt());
            if (!tmp.isEmpty())
            {
                int retVal;
                Tcl_ResetResult(tsess->tclInt());
                ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
                if (ret != TCL_ERROR)
                    if (retVal)
                        stop = TRUE;
            }
            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
unloadmodule
Modu TCL jest wyadowywany.
*/
void TclEventHandler::unloadModule()
{
    kdebug("TCL Event: unloadModule()\n");
    int ret;
    QString filter, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("unloadmodule"))
        {
            ret = Tcl_Eval(tsess->tclInt(), it.data().latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
usl lista[acuch]
Przysza z serwera lista kontaktw. Format listy jest identyczny do tego z pliku z list.
*/
void TclEventHandler::userlistReceived(struct gg_event* e)
{
    kdebug("TCL Event: userlistReceived(%s)\n", e->event.userlist.reply);
    int ret;
    QString code, filter, key, reply = QString::fromUtf8(e->event.userlist.reply);
    Tcl_DString ds;
    Tcl_DStringInit(&ds);
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("userlist"))
        {
            Tcl_DStringAppendElement(&ds, it.data().latin1());

            QCString utf;
            Tcl_Obj* result;

            utf = reply.utf8();
            result = Tcl_NewStringObj((const char* const)utf, utf.length());
            Tcl_DStringAppendElement(&ds, Tcl_GetString(result));

            ret = Tcl_Eval(tsess->tclInt(), Tcl_DStringValue(&ds));
            Tcl_DStringFree(&ds);

            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
usladd UIN[liczba]
Nowy kontakt zosta dodany do listy.
*/
void TclEventHandler::userlist_userAdded(const UserListElement& added)
{
    kdebug("TCL Event: userlist_userAdded(%d)\n", added.uin);
    int ret;
    QString filter, code, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("usladd"))
        {
            code = it.data() + " " + QString::number(added.uin);
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
usldns UIN[liczba]
DNS dla UIN'a jest gotowy.
*/
void TclEventHandler::userlist_dnsNameReady(UinType _uin)
{
    kdebug("TCL Event: userlist_dnsNameReady(%d)\n", _uin);
    int ret;
    QString filter, code, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("usldns"))
        {
            code = it.data() + " " + QString::number(_uin);
            ret = Tcl_Eval(tsess->tclInt(), code.latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}

/* Tcl Event
uslmod
Lista kontaktw zostaa zmodyfikowana (conajmniej jeden wpis).
*/
void TclEventHandler::userlist_modified()
{
    kdebug("TCL Event: userlist_modified()\n");
    int ret;
    QString filter, key;
    QMap<QString,QString>::Iterator it;
    for( it = tsess->tclEventList.begin(); it != tsess->tclEventList.end(); it++ )
    {
        key = QStringList::split(":", it.key()).first();
        filter = key;
        if (!filter.compare("uslmod"))
        {
            ret = Tcl_Eval(tsess->tclInt(), it.data().latin1());
            if (ret == TCL_ERROR)
                tsess->error(Tcl_GetStringResult(tsess->tclInt()));

            Tcl_ResetResult(tsess->tclInt());
        }
    }
}
